package com.parksong.basics.exception;

import com.alibaba.fastjson.JSON;
import com.parksong.basics.beans.ResultBean;
import com.parksong.basics.util.constants.ReturnStatusCodeConstants;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpMediaTypeException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @ClassName GlobalExceptionHandler
 * @Description 全局异常处理
 * @Author kael
 * @Date 2019/9/25 0025 下午 16:07
 * @Version 1.0
 **/
@ControllerAdvice
@RestController
@Slf4j
public class GlobalExceptionHandler {

    /**
     * 登录异常
     * @param ex
     * @return
     */
    @ExceptionHandler(NotLoginException.class)
    @ResponseStatus(value = HttpStatus.OK)
    public ResultBean<?> handleMethodArgumentNotValidExceptionHandler(NotLoginException ex) {
        ResultBean<?> result = new ResultBean();
        result.setMsg(ex.getMessage());
        result.setCode(ReturnStatusCodeConstants.NO_LOGIN);
        ex.printStackTrace();
        return result;
    }

    /**
     * 业务异常
     * @param exception
     * @return
     */
    @ExceptionHandler(value = BusinessException.class)
    @ResponseStatus(HttpStatus.OK)
    public ResultBean<?> sysLoginExceptionHandler(BusinessException ex) {
        ResultBean<?> result = new ResultBean();
        result.setMsg(ex.getMessage());
        result.setCode(ReturnStatusCodeConstants.FAIL);
        ex.printStackTrace();
        return result;
    }


    /**
     * 空异常
     * @param exception
     * @return
     */
    @ExceptionHandler(value = NullPointerException.class)
    @ResponseStatus(HttpStatus.OK)
    public ResultBean<?> httpMessageNotReadableException(NullPointerException ex) {
        ResultBean<?> result = new ResultBean();
        result.setMsg("非法入参");
        result.setCode(ReturnStatusCodeConstants.Unknown);
        ex.printStackTrace();
        return result;
    }

    /**
     * HTTP解析请求参数异常
     * @param exception
     * @return
     */
    @ExceptionHandler(value = HttpMessageNotReadableException.class)
    @ResponseStatus(HttpStatus.OK)
    public ResultBean<?> httpMessageNotReadableException(HttpMessageNotReadableException ex) {
        ResultBean<?> result = new ResultBean();
        result.setMsg("HTTP解析请求参数错误");
        result.setCode(ReturnStatusCodeConstants.Unknown);
        ex.printStackTrace();
        return result;
    }

    /**
     * HTTP
     * @param exception
     * @return
     */
    @ExceptionHandler(value = HttpMediaTypeException.class)
    @ResponseStatus(HttpStatus.OK)
    public ResultBean<?> httpMediaTypeException(HttpMediaTypeException ex) {
        ResultBean<?> result = new ResultBean();
        result.setMsg("HTTP请求出错");
        result.setCode(ReturnStatusCodeConstants.Unknown);
        ex.printStackTrace();
        return result;
    }

    /**
     * 默认的异常处理
     * @param exception
     * @return
     */
    @ExceptionHandler(value = Exception.class)
    @ResponseStatus(HttpStatus.OK)
    public ResultBean<?> exceptionHandler(Exception ex) {
        ResultBean<?> result = new ResultBean();
        result.setMsg(ex.getMessage());
        result.setCode(ReturnStatusCodeConstants.Unknown);
        ex.printStackTrace();
        return result;
    }


    /**
     * 时间处理异常
     * @param ex
     * @return
     */
    @ExceptionHandler(value = ParseException.class)
    @ResponseStatus(HttpStatus.OK)
    public ResultBean<?> exceptionHandler(ParseException ex) {
        ResultBean<?> result = new ResultBean();
        result.setMsg("时间处理失败");
        result.setCode(ReturnStatusCodeConstants.Unknown);
        ex.printStackTrace();
        return result;
    }
}
