package com.parksong.basics.util.ftp;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * @ClassName FtpUtil
 * @Description ftp服务器操作
 * @Author kael
 * @Date 2019/9/20 0020 下午 17:13
 * @Version 1.0
 **/
@Slf4j
public class FtpUtil {
    /**
     * 上传文件到ftp服务器
     * @param file
     * @param url
     * @param port
     * @param username
     * @param password
     * @return
     * @throws IOException
     */
    public static boolean uploadToFtp(File file, String url, Integer port, String username, String password) throws IOException {
        FTPClient ftpClient = new FTPClient();
        try {
            //连接ftp服务器 参数填服务器的ip
            ftpClient.connect(url,port);

            //进行登录 参数分别为账号 密码
            ftpClient.login(username, password);
            log.info("【登录到ftp服务器】：{}，【文件名】：{}",url,file.getName());
            int reply = ftpClient.getReplyCode();
            if (!FTPReply.isPositiveCompletion(reply)) {
                ftpClient.disconnect();
                return false;
            }

            //设置文件类型为二进制文件
            ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
            //改成被动模式（主动模式上传过程无响应，改成被动模式）
            ftpClient.enterLocalPassiveMode();
            log.info("【开始上传】");
            //上传文件 参数：上传后的文件名，输入流
            boolean flse = ftpClient.storeFile(file.getName(), new FileInputStream(file));
            log.info("【上传结果】：{}",flse);
            if (!flse) {
                return false;
            }
            log.info("【上传ftp服务器】：{}，【文件名】：{} ",url,file.getName());
            ftpClient.logout();

        }catch (Exception e){
            e.printStackTrace();
        }
        if (ftpClient.isConnected()) {
            try {
                ftpClient.disconnect();
            } catch (IOException ioe) {
            }
        }
        return true;
    }
}
