package com.parksong.basics.util.redis;

import com.alibaba.fastjson.JSON;
import com.parksong.basics.beans.StoreReqLimit;
import com.parksong.basics.util.configurations.SpringContextUtil;
import com.parksong.beans.config.PlatformConfig;
import com.parksong.beans.express.ExpressDeptAccount;
import com.parksong.beans.express.ExpressRule;
import com.parksong.beans.express.ExpressType;
import com.parksong.beans.store.Configuration;
import com.parksong.beans.store.Store;
import com.parksong.beans.store.StoreShelf;
import com.parksong.beans.store.StoreSmsTemplate;
import com.parksong.beans.user.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.context.ContextLoader;
import org.springframework.web.context.WebApplicationContext;

import java.util.List;
import java.util.Set;


/**
 * 缓存数据的操作类
 * @author Administrator
 *
 */
@Service
public class CacheTool {

	@Autowired
	public JedisClient jedisClient;
	
	private String jedisKey;
	private Class className;
	
	private static final String CACHE_NAMESPACE = "haliaeetus:";
    private static final String CACHB_DX = "smsSender";
	private static final String SMSCOUNT = "SMSCOUNT";
	private static final String PDD = "pdd";
	private static final String DELIVERY_STATUS = "delivery-status";
	private static final String DELIVERY_SIGN_STATUS = "delivery-sign-status";
	private static final String DELIVERY_SHELFCODE = "delivery-shelfcode:";
	private static final String JI_TU = "JITU:";
//	private static final SessionBusiness storeBusiness = SessionBusiness.init(SessionBusiness.MEMCACHED,"S");
//	private static final SessionBusiness expressBusiness = SessionBusiness.init(SessionBusiness.MEMCACHED,"E");
//	private static final SessionBusiness typeBusiness = SessionBusiness.init(SessionBusiness.MEMCACHED,"T");
	
	public CacheTool(String Hashkey, Class className){
		this.jedisKey = Hashkey;
		this.className = className;
	}

    public CacheTool(String Hashkey){
        this.jedisKey = Hashkey;
    }

	private static CacheTool storeTool = new CacheTool(CACHE_NAMESPACE + "S", Store.class);
	private static CacheTool userTool = new CacheTool(CACHE_NAMESPACE + "U", User.class);
    private static CacheTool configurationTool = new CacheTool(CACHE_NAMESPACE + "O", Configuration.class);
	private static CacheTool storeShelfTool = new CacheTool(CACHE_NAMESPACE + "SS", StoreShelf.class);
	private static CacheTool shelfTool = new CacheTool(CACHE_NAMESPACE + "ST", StoreShelf.class);
	private static CacheTool limitTool = new CacheTool(CACHE_NAMESPACE + "LI", StoreReqLimit.class);
	private static CacheTool expressRuleTool = new CacheTool(CACHE_NAMESPACE + "ER", ExpressRule.class);
	private static CacheTool storeModelTool = new CacheTool(CACHE_NAMESPACE + "SMODEL",StoreSmsTemplate.class);
	private static CacheTool expressTool = new CacheTool(CACHE_NAMESPACE + "E",ExpressType.class);
	private static CacheTool smsFlagTool = new CacheTool(CACHB_DX );
	private static CacheTool deliveryStatusTool = new CacheTool(DELIVERY_STATUS);
	private static CacheTool deliverySignStatusTool = new CacheTool(DELIVERY_SIGN_STATUS);
	private static CacheTool platformConfigTool = new CacheTool(CACHE_NAMESPACE + "pfc", PlatformConfig.class);
	private static CacheTool expressAccountTool = new CacheTool(CACHE_NAMESPACE + "EA", ExpressDeptAccount.class);
	private static CacheTool jiTuCacheTool = new CacheTool(JI_TU);
	private static CacheTool shelfcodeCacheTool = new CacheTool(DELIVERY_SHELFCODE);


	public CacheTool(){
		
	}

	public static CacheTool initStore() {
		return storeTool;
	}
	public static CacheTool initUser(){
		 return userTool;    
	}
	public static CacheTool initSMSFlag(){
		return smsFlagTool;
	}
    public static CacheTool initConfiguration() {
        return configurationTool;
    }
	public static CacheTool initStoreShelf(){
		return storeShelfTool;
	}
	public static CacheTool initShelf(){
		return shelfTool;
	}
	public static CacheTool initLimit() {
		return limitTool;
	}
	public static CacheTool initStoreModel(){ return storeModelTool; }
	public static CacheTool initExpressRule(){
		return expressRuleTool;
	}
	public static CacheTool initExpress(){
		return expressTool;
	}
	public static CacheTool initDeliveryStatusTool(){
		return deliveryStatusTool;
	}

	public static CacheTool initDeliverySignStatusTool(){
		return deliverySignStatusTool;
	}

	public static CacheTool initPlatformConfig(){
		return platformConfigTool;
	}
	
	public static CacheTool initExpressAccountTool(){
	    return expressAccountTool;
	}

    public static CacheTool initJiTuCacheTool() {
		return jiTuCacheTool;
    }

    public static CacheTool initShelfcodeCacheTool() {return shelfcodeCacheTool;}

    /**
	 * 根据key获取对象
	 */
	public Object getBean(String key ){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		String json = jedisClient.hget(this.jedisKey, key);
		if("0".equals(key)){
			//map or list
			return json;
		}else{
			return JSON.parseObject(json, this.className);
		}
	}

	/**
	 * 根据key获取对象
	 * @param <T>
	 */
	public <T> List<T> getBean(String key, Class<T> clazz){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		String json = jedisClient.hget(this.jedisKey, key);
		return JSON.parseArray(json, clazz);

	}



	/**
	 * 保存被序列化后的对象
	 */
	public boolean saveBean(String key, Object value){
		long ret = -1;
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}

		if(null == value){
			//删除操作
			ret = jedisClient.hdel(this.jedisKey, key);
		}
		else{
			String json = JSON.toJSONString(value);
			ret = jedisClient.hset(this.jedisKey, key, json);
		}

		return ret == 0 ? true: false;
	}

	/**
	 * 根据key获取String
	 */
	public String getString(String key ){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		return jedisClient.hget(this.jedisKey, key);
	}

	/**
	 * 自增
	 * @param key
	 * @return
	 */
	public Long incrBy(String key,Integer num){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		return jedisClient.incrBy(this.jedisKey+":"+key,num);
	}

    /**
     * 根据key获取对象String
     */
    public String getStringKey(String key){
        if(null == jedisClient){
            jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
        }
        return jedisClient.get(this.jedisKey+":"+key);
    }

    /**
     * set对象String
     */
    public void setStringKey(String key,String value,Integer dataTime){
        long ret = -1;
        if(null == jedisClient){
            jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
        }
        jedisClient.set(this.jedisKey+":"+key, value, dataTime);
    }

    /**
     * set对象Sting
     */
    public void setStringKey(String key,String value){
        long ret = -1;
        if(null == jedisClient){
            jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
        }
        jedisClient.set(this.jedisKey+":"+key, value);
    }

	/**
	 * 获取所有key
	 * @return
	 */
    public Set<byte[]> getKeys(){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		return jedisClient.keys(this.jedisKey+"*");
	}

	/**
	 * 根据key删除数据
	 * @return
	 */
	public void del(String key){
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		jedisClient.del(this.jedisKey+":"+key);
	}

	/**
	 * set对象String并获取过期时间
	 */
	public void setStringandDatakey(String key,String value,Integer dataTime){
		long ret = -1;
		if(null == jedisClient){
			jedisClient = (JedisClient) SpringContextUtil.getBean("jedisClient");
		}
		jedisClient.set(this.jedisKey+":"+key, value, dataTime);
	}

	public String getStringByFullKey(String key){
		if(null == jedisClient){
			WebApplicationContext wac = ContextLoader.getCurrentWebApplicationContext();
			jedisClient = (JedisClient) wac.getBean("jedisClient");
		}
		return jedisClient.get(key);
	}

}
