package com.parksong.basics.util.redis;

import java.util.Map;
import java.util.Set;

/**
 * Created by Administrator on 2016/3/13.
 */
public interface JedisClient {
    /**
     * 获取缓存
     * @param key
     * @return
     */
    String get(String key);

    byte[] get(byte[] key);

    /**
     * 设置缓存
     * @param key
     * @param value
     * @return
     */
    String set(String key, String value);

    /**
     * 设置缓存
     * @param key
     * @param value
     * @param expire 过去时间
     * @return
     */
    String set(String key, String value, int expire);

    String set(byte[] key, byte[] value);

    String set(byte[] key, byte[] value, int expire);

    /**
     * 哈希 获取缓存
     * @param hkey
     * @param key
     * @return
     */
    String hget(String hkey, String key);
    
    /**
     * 哈希 获取哈希表缓存
     * @param hkey
     * @param key
     * @return
     */
    Map<String,String> hgetall(String hkey);

    /**
     * 哈希 设置缓存
     * @param hkey
     * @param key
     * @param value
     * @return
     */
    long hset(String hkey, String key, String value);

    /**
     *获取自增值
     * @param key
     * @return
     */
    long incr(String key);

    /**
     *自增固定值
     * @param key
     * @return
     */
    long incrBy(String key, Integer num);

    /**
     *设置有效期
     * @param key
     * @param second
     * @return
     */
    long expire(String key, int second);

    /**
     *获取有效期
     * @param key
     * @return
     */
    long ttl(String key);

    /**
     * 删除缓存
     * @param key
     * @return
     */
    long del(String key);

    long del(byte[] key);

    /**
     * 删除哈希 缓存
     * @param hkey
     * @param key
     * @return
     */
    long hdel(String hkey, String key);

    Set<byte[]> keys(String pattern);

    /**
     * 添加集合元素
     * @param key
     * @return
     */
    public Long sadd(String key, String... members);
    
    /**
     * 集合元素查看
     * @param key
     * @return
     */
    public Set<String> smembers(String key);
    
    /**
     * 删除集合元素
     * @param key
     * @return
     */
    public Long srem(String key, String... members);
    
    /**
     * 查看两个集合并集
     * @param keys(集合名称1，集合名称2)
     * @return
     */
    public Set<String> sunion(String... keys);
    
    /**
     * 查看两个集合交集 
     * @param keys(集合名称1，集合名称2)
     * @return
     */
    public Long sinter(String dstkey, String... keys);
    
    /**
     * 查看两个集合差集 
     * @param keys(集合名称1，集合名称2)
     * @return
     */
    public Set<String> sdiff(String... keys);
    
    /**
     * 刷新数据
     */
    void flushDB();

    Long dbSize();
}
