package com.parksong.controllers.delivery;

import com.parksong.basics.api.ApiVersion;
import com.parksong.basics.api.Limiting;
import com.parksong.basics.beans.BatchDataMsg;
import com.parksong.basics.beans.PageResultBean;
import com.parksong.basics.beans.ResultBean;
import com.parksong.basics.controllers.BasicsController;
import com.parksong.basics.filter.ApiFilter;
import com.parksong.beans.delivery.*;
import com.parksong.beans.express.ExpressType;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.services.delivery.DeliverysService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.ParseException;
import java.util.List;

/**
 * @ClassName DeliveryController
 * @Description 业务表controller
 * @Author kael
 * @Date 2019/9/6 0006 下午 14:24
 * @Version 1.0
 **/
@RestController
@ApiVersion(1)
@RequestMapping("/{version}/delivery")
public class DeliverysController extends BasicsController {

    @Autowired
    private DeliverysService deliverysService;

    /**
     * 签收回传查询
     * @param filter
     * @return
     */
    @RequestMapping(value ="/app/querySignReturn", method = {RequestMethod.POST})
    @ResponseBody
    @ApiVersion(1)
    @Limiting
    public PageResultBean<List<Delivery>> querySignReturn(@RequestBody ApiFilter<DeliveryFilter> filter){
        return new PageResultBean<List<Delivery>>(deliverysService.querySignReturn(filter.getData()),deliverysService.querySignReturnCount(filter.getData()));
    }

    /**
     * 签收回传通过id
     * @param filter
     * @return
     */
    @RequestMapping(value ="/app/signReturn", method = {RequestMethod.POST})
    @ResponseBody
    @ApiVersion(1)
    public ResultBean<List<BatchDataMsg>> signReturnById(@RequestBody ApiFilter<DeliveryFilter> filter){
        return new ResultBean<List<BatchDataMsg>>(deliverysService.signReturnByIds(filter.getData()));
    }

    /**
     * 批量入库
     * @param filter
     * @return
     */
    @RequestMapping(value ="/app/batchStorage", method = {RequestMethod.POST})
    @ResponseBody
    @ApiVersion(1)
    public ResultBean<List<BatchDataMsg>> batchStorage(@RequestBody ApiFilter<DeliveryFilter> filter,@PathVariable String version){
        return new ResultBean<List<BatchDataMsg>>(deliverysService.batchStorageDelivery(filter.getData(),filter.getAppVersion(),version));
    }

    /**
     * 入库
     * @param filter
     * @return
     */
    @RequestMapping(value ="/app/storage", method = {RequestMethod.POST})
    @ResponseBody
    @ApiVersion(1)
    public ResultBean<Boolean> storageDelivery(@RequestBody ApiFilter<DeliveryFilter> filter,@PathVariable String version){
        return new ResultBean<>(deliverysService.storageDelivery(filter.getData(),filter.getAppVersion(),version));
    }

    /**
     * 入库--新
     * @param filter
     * @return
     */
    @RequestMapping(value ="/app/storageNew", method = {RequestMethod.POST})
    @ResponseBody
    @ApiVersion(2)
    public ResultBean<Delivery> storageDeliveryNew(@RequestBody ApiFilter<DeliveryFilter> filter,@PathVariable String version){
        return new ResultBean<>(deliverysService.storageDeliveryNew(filter.getData(),filter.getAppVersion(),version));
    }

    /**
     * 出库，签收
     * @param filter
     * @return
     */
    @RequestMapping(value ="/app/signflag", method = {RequestMethod.POST})
    @ResponseBody
    @ApiVersion(1)
    public ResultBean<List<BatchDataMsg>> signflag(@RequestBody ApiFilter<DeliveryFilter> filter,@PathVariable String version){
        return new ResultBean<List<BatchDataMsg>>(deliverysService.batchSignflag(filter.getData(),filter.getAppVersion(),version));
    }

    /**
     * 查询快件
     * @param filter
     * @return
     */
    @RequestMapping(value ="/app/queryDelivery", method = {RequestMethod.POST})
    @ResponseBody
    @ApiVersion(1)
    @Limiting
    public PageResultBean<List<Delivery>> queryDelivery(@RequestBody ApiFilter<DeliveryFilter> filter) throws ParseException {
        ResponsePage responsePage = deliverysService.queryDeliveryForPage(filter.getData());
        return new PageResultBean<List<Delivery>>(responsePage.getList(),responsePage.getTotal());
    }

    /**
     * 查询快件统计
     * @param filter
     * @return
     */
    @RequestMapping(value ="/app/queryDStatistics", method = {RequestMethod.POST})
    @ResponseBody
    @ApiVersion(1)
    public PageResultBean<DeliveryStatistics> queryDstatistics(@RequestBody ApiFilter<DeliveryFilter> filter) throws ParseException {
        return new PageResultBean<DeliveryStatistics>(deliverysService.queryDStatistics(filter.getData()));
    }

    /**
     * 通过面单号得到各商家的相关信息（获取代发短信平台和电话号码）
     * @param filter
     * @return
     */
    @RequestMapping(value ="/app/getInfoByExpCode", method = {RequestMethod.POST})
    @ResponseBody
    @ApiVersion(1)
    public ResultBean<GetInfoByExpCodeBean> getInfoByExpCode(@RequestBody ApiFilter<DeliveryFilter> filter,@PathVariable String version) throws Exception {
        return new ResultBean<GetInfoByExpCodeBean>(deliverysService.getInfoByExpCode(filter.getData(),filter.getAppVersion(),version));
    }


    /**
     * 修改快递信息
     * @param filter
     * @return
     */
    @RequestMapping(value ="/app/modify", method = {RequestMethod.POST})
    @ResponseBody
    @ApiVersion(1)
    public ResultBean<Boolean> modify(@RequestBody ApiFilter<DeliveryFilter> filter,@PathVariable String version){
        return new ResultBean<>(deliverysService.modify(filter.getData(), filter.getAppVersion(), version));
    }

    /**
     * 根据快递单号查询快递公司
     */
    @RequestMapping(value ="/app/getExpressByExpCode", method = {RequestMethod.POST})
    public ResultBean<ExpressType> getExpressByExpCode(@RequestBody ApiFilter<DeliveryFilter> filterApiFilter, @PathVariable String version){
        return new ResultBean<>(deliverysService.getExpressByExpCode(filterApiFilter, version));
    }

    /**
     * 查询出入库信息
     */
    @RequestMapping(value = "/app/queryInOutBound", method = {RequestMethod.POST})
    public ResultBean<DeliveryInOutBoundDto> queryInOutBound(@RequestBody ApiFilter<DeliveryFilter> filter){
        DeliveryInOutBoundDto dto = deliverysService.queryInOutBound(filter);
        if (dto != null) {
            return new ResultBean<>(dto);
        }
        ResultBean resultBean = new ResultBean();
        resultBean.setCode(-1);
        resultBean.setMsg("没查询到相关数据");
        return resultBean;
    }

    /**
     * 统计出库率
     * @param filter
     * @return
     */
    @PostMapping(value ="/app/outboundRate")
    @ApiVersion(1)
    public PageResultBean<StatisticsOutboundRate> outboundRate(@RequestBody ApiFilter<DeliveryFilter> filter){
        return new PageResultBean<>(deliverysService.outboundRate(filter.getData()));
    }

    /**
     * 出库率数据导出
     * @param filter
     * @return
     */
    @PostMapping(value ="/app/export")
    @ApiVersion(1)
    public void export(HttpServletResponse response, @RequestBody DeliveryFilter filter) throws IOException {
        deliverysService.export(response, filter);
    }

    /**
     * 根据收件人电话号码查询收件人信息
     */
    @PostMapping(value ="/app/getConsigneeInfoByPhone")
    public ResultBean<List<PhoneLibDto>> getConsigneeInfoByPhone(@RequestBody ApiFilter<DeliveryFilter> filter){
        return new ResultBean<>(deliverysService.getConsigneeInfoByPhone(filter.getData()));
    }

    /**
     * 获取快递详情接口
     */
    @PostMapping(value ="/app/expressDetailInfo")
    public ResultBean<Delivery> expressDetailInfo(@RequestBody ApiFilter<DeliveryFilter> filter){
        return new ResultBean<>(deliverysService.expressDetailInfo(filter.getData()));
    }

    /**
     * 添加寄件人信息
     */
    @PostMapping(value ="/app/addSenderInfo")
    public ResultBean<Boolean> addSenderInfo(@RequestBody ApiFilter<DeliveryFilter> filter){
        return new ResultBean<>(deliverysService.addSenderInfo(filter.getData()));
    }
}
