package com.parksong.dao.delivery;

import com.parksong.basics.dao.BasicDao;
import com.parksong.beans.delivery.*;
import com.parksong.filter.delivery.DeliveryFilter;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author kael
 */
@Repository
public interface DeliverysDao extends BasicDao<DeliveryFilter, Delivery> {

    /**
     * 通过id或者快递单号查询数据
     *
     * @param filter
     * @return
     */
    Delivery queryDeliveryByIdOrExpCode(@Param("filter") DeliveryFilter filter);

    /**
     * 查询总表数据，返回国安需要数据
     *
     * @param filter
     * @return
     */
    List<GuoanFtpData> queryDeliveryByGuoan(DeliveryFilter filter);

    /**
     * 修改订单发送状态
     *
     * @param filter
     * @return
     */
    Integer updateDeliverySmsStatus(DeliveryFilter filter);

    /**
     * 通过id集合查询订单数据返回map
     *
     * @param filter
     * @return
     */
    @MapKey("id")
    Map<Integer, Delivery> queryMapByDeliveryIds(DeliveryFilter filter);

    /**
     * 新增订单时插入总表
     *
     * @param delivery
     * @return
     */
    Integer insertTotalDelivery(Delivery delivery);

    /**
     * 插入统计表
     *
     * @param baseStatToday
     * @return
     */
    Integer saveOrUpdateStatToday(BaseStatToday baseStatToday);

    /**
     * 每天统计数据
     *
     * @param baseStatToday
     * @return
     */
    Integer pushTodayToYesterday(BaseStatToday baseStatToday);

    /**
     * 更新总表信息
     *
     * @param delivery
     * @return
     */
    Integer updateDeliveryToday(Delivery delivery);

    /**
     * 通过电话号码查询数据
     *
     * @param filter
     * @return
     */
    List<Delivery> getDeliveryByPhones(DeliveryFilter filter);

    Integer queryIdFromMasterDelivery(Delivery delivery);

    Integer updateMasterDelivery(@Param("entity") Delivery entity, @Param("param") Delivery param);

    int getUnsignExpCount(DeliveryFilter deliveryFilter);

    List<Delivery> queryMasterDelivery(Delivery delivery);

    List<StatisticsOutboundRate> todayData(@Param("storeId") Integer storeId, @Param("todayTime") String todayTime, @Param("expId") Integer expId);

    List<String> historyTime(@Param("storeId") Integer storeId, @Param("beginTime") String beginTime, @Param("endTime") String endTime);

    StatisticsOutboundRate getExportData(@Param("storeId") Integer storeId, @Param("time") String time);

    List<PhoneLibDto> getConsigneeInfoByPhone(@Param("filter") DeliveryFilter filter);

    Delivery expressDetailInfo(@Param("deliveryId") Integer deliveryId, @Param("storeId") Integer storeId);

    int selectDelivery(@Param("storeId") Integer storeId, @Param("shelfCode") String shelfCode);

    List<ExpressOperation> selectExpressOperationRecord(@Param("storeId") Integer storeId,@Param("expcode") String expcode);

    void addSenderInfo(@Param ("filter") DeliveryFilter filter);
}
