package com.parksong.factory.infoplatform.realize;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Strings;
import com.parksong.basics.util.basic.Md5Util;
import com.parksong.beans.config.PlatformConfig;
import com.parksong.beans.delivery.GetInfoByExpCodeBean;
import com.parksong.beans.express.ExpressType;
import com.parksong.beans.platform.cainiao.CaiNiaoRequest;
import com.parksong.beans.platform.cainiao.CaiNiaoResponse;
import com.parksong.beans.platform.cainiao.CaiNiaoWaybillQuery;
import com.parksong.factory.infoplatform.BaseInfoPlatform;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.services.config.PlatformConfigService;
import com.parksong.util.Http.OkHttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @ClassName AlbbPlatfrom
 * @Description 阿里短信服务
 * @Author wsc
 * @Date 2019/11/27 下午 16:59
 * @Modefy
 * @Version 1.0
 */
@Component("CaiNiao")
@Slf4j
public class CaiNiaoPlatform implements BaseInfoPlatform {
    @Autowired
    PlatformConfigService platformConfigService;

    @Override
    public GetInfoByExpCodeBean getInfoByExpCode(DeliveryFilter filter) {
        String logisticsInterface = JSON.toJSONString(CaiNiaoWaybillQuery.builder().waybillNo(filter.getExpCode()).stationCode(filter.getStoreId().toString()).build());
        PlatformConfig platformConfig = platformConfigService.getByPlatform("CaiNiao");
        CaiNiaoRequest caiNiaoRequest = new CaiNiaoRequest();
        caiNiaoRequest.setMsg_type("TMS_WAYBILL_EXTEND_INFO_QUERY");
        caiNiaoRequest.setLogistic_provider_id(platformConfig.getToken());
        caiNiaoRequest.setData_digest(Md5Util.caiNiaoDoSign(logisticsInterface, "utf-8", platformConfig.getAppSecret()));
        caiNiaoRequest.setLogistics_interface(logisticsInterface);
        caiNiaoRequest.setTo_code("TMS_WAYBILL_EXTEND_INFO_QUERY");
        caiNiaoRequest.setUrl(platformConfig.getUrl());
        CaiNiaoResponse caiNiaoResponse = null;
        try {
            String res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl() + "/cainiao/waybillExtendInfoQuery", caiNiaoRequest);
            caiNiaoResponse = JSONObject.parseObject(res, CaiNiaoResponse.class);
        } catch (Exception e) {
            log.error(e.toString());
        }
        if (caiNiaoResponse != null && caiNiaoResponse.getSuccess()) {
            if (!Strings.isNullOrEmpty(caiNiaoResponse.getEndOfMobileNumber())){
                return GetInfoByExpCodeBean.builder().telPhone("1811111" + caiNiaoResponse.getEndOfMobileNumber()).platform("CaiNiao").build();
            }
            return GetInfoByExpCodeBean.builder().platform("CaiNiao-B").build();
        }
        return null;
    }

    @Override
    public ExpressType getExpressByExpCode(String expCode) {
        return null;
    }
}