package com.parksong.factory.infoplatform.realize;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Strings;
import com.parksong.basics.util.redis.CacheTool;
import com.parksong.beans.config.PlatformConfig;
import com.parksong.beans.delivery.GetInfoByExpCodeBean;
import com.parksong.beans.express.ExpressType;
import com.parksong.beans.platform.jitu.JiTuRequestBean;
import com.parksong.beans.platform.jitu.JiTuResponseBean;
import com.parksong.factory.infoplatform.BaseInfoPlatform;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.services.config.PlatformConfigService;
import com.parksong.util.Http.OkHttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author liujs
 */
@Component("296")
@Slf4j
public class JiTuPlatform implements BaseInfoPlatform {
    @Autowired
    private PlatformConfigService platformConfigService;

    @Override
    public GetInfoByExpCodeBean getInfoByExpCode(DeliveryFilter filter) throws Exception {
        PlatformConfig platformConfig = platformConfigService.getByPlatform("JTSD");
        long timestamp = System.currentTimeMillis();
        String appId = platformConfig.getAppkey();
        // 正式环境
         String logisticsUrl = platformConfig.getLogisticsUrl() + "/jitu/query/mobile";
         String host = platformConfig.getUrl();

        // 测试环境
        // String logisticsUrl = "http://callback.fengzhan100.com/logistics/jitu/query/mobile";
        // String host = "https://test-cabinet.jtexpress.com.cn";

        String accessToken = getAccessToken();
        int i = 0;
        while (i < 2) {
            String url = host + "/ecs/standard/cabinet/query/mobile?access_token=" + accessToken + "&waybillId=" + filter.getExpCode();
            String res = OkHttpUtil.postJson(logisticsUrl, JiTuRequestBean.builder().url(url).waybillId(filter.getExpCode()).build());
            JiTuResponseBean jiTuResponseBean = JSONObject.parseObject(res, JiTuResponseBean.class);
            if (!jiTuResponseBean.getSucc() && ("15301009".equals(jiTuResponseBean.getCode()) || "15301010".equals(jiTuResponseBean.getCode()))) {
                accessToken = getAccessToken(true);
                i++;
                continue;
            }
            if (jiTuResponseBean.getSucc()){
                return GetInfoByExpCodeBean.builder().telPhone(jiTuResponseBean.getData().getReceiverMobilePhone()).platform("JiTu").build();
            }
            break;
        }
        return null;
    }

    @Override
    public ExpressType getExpressByExpCode(String expCode) {
        return null;
    }


    /**
     * 获取accessToken,默认在缓存中获取
     */
    private String getAccessToken(){
        return getAccessToken(false);
    }

    /**
     * refresh 是否刷新
     */
    private String getAccessToken(boolean refresh) {
        String accessToken = null;
        try {
            CacheTool cacheTool = CacheTool.initJiTuCacheTool();
            /*
             *  当refresh == true 时强制刷新,重新获取accessToken
             *  当缓存中的accessToken 为空时，重新获取accessToken
             */
            if (refresh || Strings.isNullOrEmpty((accessToken = cacheTool.getStringKey("access_token")))) {
                PlatformConfig platformConfig = platformConfigService.getByPlatform("JTSD");
                // 生产环境
                String corpid = platformConfig.getAppkey();
                String corpsecret = platformConfig.getAppSecret();
                String host = platformConfig.getUrl();

                // 测试环境
                // String host = "https://test-cabinet.jtexpress.com.cn";

                String res = OkHttpUtil.get(host + "/ecs/standard/cabinet/gettoken?corpid=" + corpid + "&corpsecret=" + corpsecret);
                JSONObject resObj = JSON.parseObject(res);
                JSONObject data = resObj.getJSONObject("data");
                accessToken = data.getString("access_token");
                cacheTool.setStringKey("access_token", accessToken, data.getInteger("expires_in"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println(accessToken);
        return accessToken;
    }

}
