package com.parksong.factory.infoplatform.realize;

import com.google.common.base.Strings;
import com.google.gson.Gson;
import com.parksong.beans.delivery.GetInfoByExpCodeBean;
import com.parksong.beans.express.ExpressType;
import com.parksong.beans.platform.enums.PddExpressEnum;
import com.parksong.beans.platform.pdd.OrderQueryInfo;
import com.parksong.beans.platform.pdd.OrderQueryResponse;
import com.parksong.beans.platform.pdd.PddGetSysParam;
import com.parksong.factory.infoplatform.BaseInfoPlatform;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.services.store.ConfigurationService;
import com.parksong.util.Http.OkHttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @ClassName PddPlatform
 * @Description 拼多多
 * @Author wsc
 * @Date 2019/11/27 0028 下午 15:45
 * @Version 1.0
 **/
@Component("PDD")
@Slf4j
public class PddPlatform implements BaseInfoPlatform {

    @Autowired
    private ConfigurationService configurationService;

    @Override
    public GetInfoByExpCodeBean getInfoByExpCode(DeliveryFilter filter) throws Exception {
        PddGetSysParam pddGetSysParam = new PddGetSysParam().invoke(configurationService);
        try{
            String res = postPddQueryPhone(filter, pddGetSysParam.getUrl(), pddGetSysParam.getPddUrl(), pddGetSysParam.getPartnerId(), pddGetSysParam.getSecret());
            if(!Strings.isNullOrEmpty(res)){
                OrderQueryResponse orderQueryResponse = new Gson().fromJson(res,OrderQueryResponse.class);
                if(orderQueryResponse.isSuccess()&&orderQueryResponse.getResult().compareTo(1)==0){
                    return GetInfoByExpCodeBean.builder().telPhone(orderQueryResponse.getPhoneNum()).platform("PDD").build();
                }
            }
        }catch (Exception e){
            log.error(e.toString());
        }
        return null;
    }

    @Override
    public ExpressType getExpressByExpCode(String expCode) {
        return null;
    }

    /**
     * 拼多多通过手机号码查询订单接口
     * @param DeliveryFilter
     * @param url
     * @param pddUrl
     * @param partnerId
     * @param secret
     * @return
     * @throws Exception
     */
    private String postPddQueryPhone(DeliveryFilter filter, String url, String pddUrl, String partnerId, String secret) throws Exception {
        OrderQueryInfo orderQueryInfo = OrderQueryInfo.builder()
                .companyCode(partnerId)
                .cabCode(filter.getStoreId().toString())
                .expressCode(PddExpressEnum.of(filter.getExpId()).getBrand())
                .mailNo(filter.getExpCode())
                .siteType(2)
                .build();
        orderQueryInfo.setApiName(pddUrl+"/express/order/"+partnerId+"/query");
        orderQueryInfo.setPartner_id(partnerId);
        orderQueryInfo.setSecret(secret);
        log.info("kael-------拼多多通过手机号码查询订单传入参数："+new Gson().toJson(orderQueryInfo));
        return OkHttpUtil.postJson(url+"/pddPhoneQuery", orderQueryInfo);
    }
}
