package com.parksong.factory.infoplatform.realize;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Strings;
import com.parksong.beans.config.PlatformConfig;
import com.parksong.beans.delivery.GetInfoByExpCodeBean;
import com.parksong.beans.express.ExpressType;
import com.parksong.beans.platform.youxiansheng.YouXianShengGetPhoneParam;
import com.parksong.beans.platform.youxiansheng.YouXianShengResult;
import com.parksong.factory.infoplatform.BaseInfoPlatform;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.services.config.PlatformConfigService;
import com.parksong.services.store.ConfigurationService;
import com.parksong.util.Http.OkHttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author LiuJS
 * @Time 2021/4/28 10:23
 * @Desc 优鲜生平台
 * @Version 1.0
 */
@Component("298")
@Slf4j
public class YouXianShengPlatform implements BaseInfoPlatform {

    @Autowired
    private ConfigurationService configurationService;

    @Autowired
    private PlatformConfigService platformConfigService;

    @Override
    public GetInfoByExpCodeBean getInfoByExpCode(DeliveryFilter filter) throws Exception {
        return youXianShengGetPhone(filter.getExpCode());
    }

    private GetInfoByExpCodeBean youXianShengGetPhone(String expCode) {
        PlatformConfig yxs = platformConfigService.getByPlatform("YXS");
        String yxsGetPhoneSign = configurationService.getConfigurationByCode("YXSGetPhoneSign").getOValue1();
        String url = yxs.getLogisticsUrl() + "/youxiansheng/static/getAddress";
        YouXianShengGetPhoneParam youXianShengGetPhoneParam = new YouXianShengGetPhoneParam();
        youXianShengGetPhoneParam.setOrderId(expCode);
        youXianShengGetPhoneParam.setSign(yxsGetPhoneSign);
        GetInfoByExpCodeBean expCodeBean = null;
        try {
            String res = OkHttpUtil.postJson(url, youXianShengGetPhoneParam);
            JSONObject jsonObject = JSON.parseObject(res);
            String consigneePhone = null;
            if (jsonObject != null && jsonObject.getString("result") != null) {
                YouXianShengResult youXianShengResult = JSONObject.parseObject(jsonObject.getString("result"), YouXianShengResult.class);
                consigneePhone = youXianShengResult.getConsigneeTel();
                if (!Strings.isNullOrEmpty(consigneePhone)){
                    expCodeBean = GetInfoByExpCodeBean.builder().telPhone(consigneePhone).platform("YXS").build();
                }
            }
        } catch (Exception e) {
            log.error(e.toString());
        }
        return expCodeBean;
    }

    @Override
    public ExpressType getExpressByExpCode(String expCode) {
        return null;
    }
}
