package com.parksong.factory.infoplatform.realize;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.parksong.beans.config.PlatformConfig;
import com.parksong.beans.delivery.GetInfoByExpCodeBean;
import com.parksong.beans.express.ExpressType;
import com.parksong.beans.platform.enums.ZtoExpressEnum;
import com.parksong.beans.platform.zto.ZtoQueryMobileRequest;
import com.parksong.beans.platform.zto.ZtoQueryMobileResponse;
import com.parksong.factory.infoplatform.BaseInfoPlatform;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.services.config.PlatformConfigService;
import com.parksong.services.store.ConfigurationService;
import com.parksong.util.Http.OkHttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @ClassName ZtoPlatform
 * @Description 中通
 * @Author wsc
 * @Date 2019/11/27 0027 下午 15:09
 * @Version 1.0
 **/
@Component("187")
@Slf4j
public class ZtoPlatform implements BaseInfoPlatform {

    @Autowired
    private ConfigurationService configurationService;

    @Autowired
    private PlatformConfigService platformConfigService;

    private static final String PLATFORM = "ZTO";

    @Override
    public GetInfoByExpCodeBean getInfoByExpCode(DeliveryFilter filter) {
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFORM);
        GetInfoByExpCodeBean getInfoByExpCodeBean = null;
        ZtoQueryMobileRequest request = ZtoQueryMobileRequest.builder()
                .companyCode(platformConfig.getToken())
                .billCode(filter.getExpCode())
                .build();
        request.setApiName(platformConfig.getUrl() + "/getMobileAndNameByBillCode");
        request.setCompanyId(platformConfig.getAppSecret());
        request.setKey(platformConfig.getAppkey());
        try {
            log.info("kael---->中通获取电话号码接口调用传入参数：" + JSON.toJSONString(request));
            String res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl() + "/ztoGetMobile", request);
            log.info("kael---->中通获取电话号码接口返回：" + res);
            if (!Strings.isBlank(res)) {
                ZtoQueryMobileResponse ztoQueryMobileResponse = JSON.parseObject(res, ZtoQueryMobileResponse.class);
                if (ztoQueryMobileResponse.getStatus() && "200".equals(ztoQueryMobileResponse.getStatusCode())) {
                    getInfoByExpCodeBean = GetInfoByExpCodeBean.builder().platform("ZTO").telPhone("1710000" + ztoQueryMobileResponse.getResult().getMobile().substring(7)).build();
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return getInfoByExpCodeBean;
    }

    @Override
    public ExpressType getExpressByExpCode(String expCode) {
        PlatformConfig platformConfig = platformConfigService.getByPlatform(PLATFORM);
        ZtoQueryMobileRequest request = ZtoQueryMobileRequest.builder()
                .companyCode(platformConfig.getToken())
                .billCode(expCode)
                .build();
        request.setApiName(platformConfig.getUrl() + "/billAnalysis");
        request.setCompanyId(platformConfig.getAppSecret());
        request.setKey(platformConfig.getAppkey());
        try {
            String res = OkHttpUtil.postJson(platformConfig.getLogisticsUrl() + "/ztoBillAnalysis", request);
            JSONObject jsonObject = JSON.parseObject(res);
            String statusCode, result;
            if (!Strings.isEmpty(statusCode = jsonObject.getString("statusCode")) && statusCode.equals("SYS000") && !Strings.isEmpty(result = jsonObject.getString("result"))) {
                ZtoExpressEnum ztoExpressEnum = ZtoExpressEnum.valueOf(result);
                ExpressType expressType = new ExpressType();
                expressType.setExpId(ztoExpressEnum.getExpressId());
                expressType.setExpName(ztoExpressEnum.getCnName());
                return expressType;
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return null;
    }
}
