package com.parksong.job;

import com.alibaba.fastjson.JSON;
import com.parksong.basics.util.basic.DateUtil;
import com.parksong.basics.util.configurations.GuoanFtpConfig;
import com.parksong.basics.util.ftp.FtpUtil;
import com.parksong.beans.delivery.GuoanFtpData;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.services.delivery.DeliverysService;
import com.parksong.util.enums.ExpressCompanyEnum;
import com.parksong.util.txt.TxtUtils;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.IJobHandler;
import com.xxl.job.core.handler.annotation.JobHandler;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @ClassName GuoanJobHandle
 * @Description 国安上传ftp文件
 * @Author kael
 * @Date 2019/9/20 0020 下午 17:01
 * @Version 1.0
 **/
@JobHandler(value = "guoanJobHandle")
@Component
@Slf4j
public class GuoanJobHandle extends IJobHandler {

    @Autowired
    private GuoanFtpConfig guoanFtpConfig;

    @Autowired
    private DeliverysService deliverysService;

    @Override
    public ReturnT<String> execute(String param) {
        long startTime = System.currentTimeMillis();
        try {
            log.info("【进入guoanJobHandle自动任务】");
            DeliveryFilter filter = new DeliveryFilter();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String time = DateUtil.addDay(sdf.format(new Date()), "yyyy-MM-dd", -1);
            String fileName = time.replaceAll("-", "") + "_235959_fz.txt";
            filter.setBeginTime(time + " 00:00:00");
            filter.setEndTime(time + " 23:59:59");
            List<GuoanFtpData> guoanFtpDataList = deliverysService.queryDeliveryByGuoan(filter);
            String basePath = TxtUtils.getResourceBasePath();
            StringBuilder json = new StringBuilder();
            for (GuoanFtpData guoanFtpData : guoanFtpDataList) {
                if (Strings.isBlank(guoanFtpData.getExpress_com())) {
                    continue;
                }
                ExpressCompanyEnum exp = ExpressCompanyEnum.valueOf(Integer.parseInt(guoanFtpData.getExpress_com()));
                if (exp == null) {
                    continue;
                }
                guoanFtpData.setExpress_com(exp.getEnValue());
                json.append(JSON.toJSONStringWithDateFormat(guoanFtpData, "yyyy-MM-dd HH:mm:ss"));
                json.append(System.getProperty("line.separator"));
            }
            TxtUtils.writeToTxt(json.toString(), basePath, fileName);
            log.info("【生成文件所在路径】：{} ，【文件名】：{}", basePath+"/guoanFile/", fileName);
            File file = new File(basePath, "guoanFile/" + fileName);
            log.info("【开始发送ftp服务器】：{} ", guoanFtpConfig.getUrl());
            boolean refult = FtpUtil.uploadToFtp(file, guoanFtpConfig.getUrl(), guoanFtpConfig.getPort(), guoanFtpConfig.getUsername(), guoanFtpConfig.getPassword());
            if (file != null) {
                log.info("【删除文件】：{} ", fileName);
                file.delete();
            }
            log.info("【guoanJobHandle自动任务结束，使用时间】：{} 毫秒", (System.currentTimeMillis() - startTime));
        } catch (Exception e) {
            e.printStackTrace();
        }

        return SUCCESS;
    }
}
