package com.parksong.services.applets.impl;

import com.parksong.basics.exception.BusinessException;
import com.parksong.beans.delivery.ResponsePage;
import com.parksong.beans.delivery.StatisticsOutboundRate;
import com.parksong.beans.express.ConsumeStatistics;
import com.parksong.beans.express.ExpressApplets;
import com.parksong.beans.express.ExpressDept;
import com.parksong.dao.applets.AppletsMapper;
import com.parksong.filter.applets.AppletsFilter;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.services.applets.AppletsService;
import com.parksong.services.delivery.DeliverysService;
import com.parksong.util.JwtUtils;
import com.parksong.util.enums.ExpressStatusEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author LiuJS
 * @Time 2021/7/6 11:12
 * @Desc
 * @Version 1.0
 */
@Service
public class AppletsServiceImpl implements AppletsService {

    @Autowired
    private AppletsMapper appletsMapper;

    @Autowired
    private DeliverysService deliverysService;

    @Override
    public String userLogin(String account, String password) throws Exception {
        if (StringUtils.isBlank(account) || StringUtils.isBlank(password)){
            throw new BusinessException("账号或者密码不能为空");
        }
        ExpressDept expressDept = appletsMapper.userLogin(account);
        if (expressDept == null){
            throw new BusinessException("账号不存在或者账号状态出现异常");
        }else {
            if (DigestUtils.md5DigestAsHex(("youbaimi" + password).getBytes()).equals(expressDept.getPassword())){
                Map<String, String> claims = new HashMap<>(3);
                claims.put("uid", expressDept.getId().toString());
                claims.put("storeid", expressDept.getStoreId().toString());
                claims.put("expid", expressDept.getExpId().toString());
                claims.put("uname", expressDept.getOwnerName());
                claims.put("uphone", expressDept.getOwnerPhone());
                return JwtUtils.createToken(claims);
            } else {
                throw new BusinessException("密码错误");
            }
        }

    }

    @Override
    public ExpressDept userDetail(AppletsFilter filter) {
        return appletsMapper.userDetail(filter.getExpressDept().getId());
    }

    @Override
    public ResponsePage expressInfo(AppletsFilter filter) {
        ResponsePage responsePage = new ResponsePage();
        // 查询前的filter数据处理
        this.queryDeliveryFilterHandle(filter);
        List<ExpressApplets> expressApplets = appletsMapper.expressInfo(filter);
        expressApplets.stream().forEach(express -> express.setStatusName(ExpressStatusEnum.valueOf(express.getStatus()).getValue()));
        responsePage.setExpressAppletsList(expressApplets);
        responsePage.setTotal(appletsMapper.selectExpressInfoCount(filter));
        return responsePage;
    }

    @Override
    public StatisticsOutboundRate expressStatistics(AppletsFilter filter) {
        DeliveryFilter deliveryFilter = new DeliveryFilter();
        deliveryFilter.setStoreId(filter.getStoreId());
        deliveryFilter.setBeginTime(filter.getStartTime());
        deliveryFilter.setEndTime(filter.getEndTime());
        deliveryFilter.setExpId(filter.getExpressDept().getExpId().intValue());
        return deliverysService.outboundRate(deliveryFilter);
    }

    @Override
    public List<ConsumeStatistics> consumeStatistics(AppletsFilter filter) {
        filter.setExpressId(filter.getExpressDept().getExpId().intValue());
        List<ConsumeStatistics> consumeStatisticsList = appletsMapper.consumeStatistics(filter);
        consumeStatisticsList.stream().forEach(consumeStatistics -> consumeStatistics.setConsumeAmount(new BigDecimal(consumeStatistics.getNumber()).multiply(consumeStatistics.getPrice())));
        return consumeStatisticsList;
    }

    private void queryDeliveryFilterHandle(AppletsFilter filter){
        filter.sortInit();
    }
}
