package com.parksong.services.delivery;

import com.parksong.basics.beans.BatchDataMsg;
import com.parksong.basics.filter.ApiFilter;
import com.parksong.beans.delivery.*;
import com.parksong.beans.express.ExpressType;
import com.parksong.filter.delivery.DeliveryFilter;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.ParseException;
import java.util.List;
import java.util.Map;

/**
 * @author kael
 */
public interface DeliverysService {
    /**
     * 签收回传查询
     * @param filter
     * @return
     */
    List<Delivery> querySignReturn(DeliveryFilter filter);

    /**
     * 签收回传总条数
     * @param filter
     * @return
     */
    Long querySignReturnCount(DeliveryFilter filter);

    /**
     * 通过ids签收回传发送
     * @param filter
     * @return
     */
    List<BatchDataMsg> signReturnByIds(DeliveryFilter filter);

    /**
     * 回传
     * @param expId
     * @param expcode
     * @param userPhone
     * @param storeId
     * @param status
     * @param s
     * @param shelfCode
     * @param platform
     */
    void signBack(Integer expId, String expcode, String userPhone, Integer storeId, String status, String s, String shelfCode,String platform);

    /**
     * 批量入库
     * @param filter
     * @param appVersion
     * @param apiVersion
     * @return
     */
    List<BatchDataMsg> batchStorageDelivery(DeliveryFilter filter,String appVersion,String apiVersion);

    /**
     * 入库
     * @param filter
     * @param appVersion
     * @param apiVersion
     * @return
     */
    boolean storageDelivery(DeliveryFilter filter, String appVersion, String apiVersion);

    /**
     * 通过面单号得到各商家的相关信息
     * @param filter
     * @param appVersion
     * @param apiVersion
     * @return
     */
    GetInfoByExpCodeBean getInfoByExpCode(DeliveryFilter filter, String appVersion, String apiVersion) throws Exception;

    /**
     * 出库（签收）
     * @param filter
     * @param appVersion
     * @param apiVersion
     * @return
     */
    List<BatchDataMsg> batchSignflag(DeliveryFilter filter,String appVersion,String apiVersion);

    /**
     * 查询快件
     * @param filter
     * @return
     * @throws ParseException
     */
    List<Delivery> queryDelivery(DeliveryFilter filter) throws ParseException;

    /**
     * 查询快件条数
     * @param filter
     * @return
     * @throws ParseException
     */
    Long queryDeliveryCount(DeliveryFilter filter) throws ParseException;

    /**
     * 查询快件统计
     * @param filter
     * @return
     * @throws ParseException
     */
    DeliveryStatistics queryDStatistics(DeliveryFilter filter) throws ParseException ;

    /**
     * 查询总表数据，返回国安需要数据
     * @param filter
     * @return
     */
    List<GuoanFtpData> queryDeliveryByGuoan(DeliveryFilter filter);

    /**
     * 修改订单短信状态
     * @param filter
     * @return
     */
    Integer updateDeliverySmsStatus(DeliveryFilter filter);

    /**
     * 通过id集合查询订单数据返回map
     * @param deliveryIds
     * @param storeId
     * @return
     */
    Map<Integer,Delivery> queryMapByDeliveryIds(List<Integer> deliveryIds, Integer storeId);

    /**
     * 插入订单总表
     * @param delivery
     * @return
     */
    Integer insertTotalDelivery(Delivery delivery);

    /**
     * 插入统计结果
     * @param delivery
     * @return
     */
    Integer saveOrUpdateStatToday(Delivery delivery);

    /**
     * 每天统计数据
     * @param baseStatToday
     * @return
     */
    Integer pushTodayToYesterday(BaseStatToday baseStatToday);

    /**
     * 更新总表
     * @param delivery
     * @return
     */
    Integer updateDeliveryToday(Delivery delivery);

    /**
     * 通过电话号码获取为出库的订单数据
     * @param filter
     * @return
     */
    List<Delivery> getDeliveryByPhones(DeliveryFilter filter);

    Integer queryIdFromMasterDelivery(Delivery delivery);

    Boolean modify(DeliveryFilter data, String appVersion, String version);

    Integer updateMasterDelivery(Delivery entity, Delivery param);

    ResponsePage queryDeliveryForPage(DeliveryFilter deliveryFilter) throws ParseException;

    ExpressType getExpressByExpCode(ApiFilter<DeliveryFilter> filterApiFilter, String version);

    DeliveryInOutBoundDto queryInOutBound(ApiFilter<DeliveryFilter> filter);

    StatisticsOutboundRate outboundRate(DeliveryFilter filter);

    void export(HttpServletResponse response, DeliveryFilter filter) throws IOException;

    List<PhoneLibDto> getConsigneeInfoByPhone(DeliveryFilter filter);

    Delivery storageDeliveryNew(DeliveryFilter data, String appVersion, String version);

    Delivery expressDetailInfo(DeliveryFilter filter);

    boolean addSenderInfo(DeliveryFilter filter);
}
