package com.parksong.services.delivery.impl;

import com.parksong.basics.services.BasicsService;
import com.parksong.beans.delivery.BaseStatToday;
import com.parksong.beans.delivery.Delivery;
import com.parksong.beans.delivery.DeliveryScanBatch;
import com.parksong.beans.delivery.DeliveryStatus;
import com.parksong.services.delivery.DeliveryAsyncService;
import com.parksong.services.delivery.DeliveryScanBatchService;
import com.parksong.services.delivery.DeliveryStatusService;
import com.parksong.services.delivery.DeliverysService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Future;

/**
 * @ClassName DeliveryAsyncServiceImpl
 * @Description 业务表使用线程池处理的方法
 * @Author kael
 * @Date 2019/10/11 0011 上午 9:35
 * @Version 1.0
 **/
@Service
public class DeliveryAsyncServiceImpl extends BasicsService implements DeliveryAsyncService {

    @Autowired
    private DeliverysService deliverysService;

    @Autowired
    private DeliveryStatusService deliveryStatusService;

    @Autowired
    private DeliveryScanBatchService deliveryScanBatchService;

    @Async("taskExecutor")
    @Override
    public CompletableFuture<Integer> insertTotalDelivery(Delivery delivery){
        //插入统计
        deliverysService.saveOrUpdateStatToday(delivery);
        //插入总表
        return CompletableFuture.completedFuture(deliverysService.insertTotalDelivery(delivery));
    }

    @Async("taskExecutor")
    @Override
    public Integer updateDeliveryToday(Delivery delivery){
        //插入总表
        return deliverysService.updateDeliveryToday(delivery);
    }


    @Async("taskExecutor")
    @Override
    public Integer insertTotalDeliveryStatus(DeliveryStatus deliveryStatus){
        //插入总订单状态表
        return deliveryStatusService.insertTotalDeliveryStatus(deliveryStatus);
    }

    @Async("taskExecutor")
    @Override
    public Integer pushTodayToYesterday(BaseStatToday baseStatToday){
        return deliverysService.pushTodayToYesterday(baseStatToday);
    }

    @Async("taskExecutor")
    @Override
    public Integer insertTotalDeliveryScanBatch(DeliveryScanBatch deliveryScanBatch){
        return deliveryScanBatchService.insertTotalDeliveryScanBatch(deliveryScanBatch);
    }

    @Async("taskExecutor")
    @Override
    public CompletableFuture<Integer> updateMasterDelivery(Delivery entity, Delivery delivery) {
        return CompletableFuture.completedFuture(deliverysService.updateMasterDelivery(entity, delivery));
    }
}
