package com.parksong.services.delivery.impl;

import com.parksong.basics.services.BasicsService;
import com.parksong.basics.util.basic.ClassCompareUtil;
import com.parksong.beans.delivery.Delivery;
import com.parksong.beans.delivery.DeliveryRunningWater;
import com.parksong.beans.sms.DeliveryDto;
import com.parksong.dao.delivery.DeliveryRunningWaterDao;
import com.parksong.services.delivery.DeliveryRunningWaterService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

/**
 * @ClassName DeliveryRunningWaterServiceImpl
 * @Description 订单流水
 * @Author kael
 * @Date 2019/9/16 0016 下午 17:39
 * @Version 1.0
 **/
@Service
public class DeliveryRunningWaterServiceImpl extends BasicsService implements DeliveryRunningWaterService {

    @Autowired
    private DeliveryRunningWaterDao deliveryRunningWaterDao;

    @Override
    public Integer insert(DeliveryRunningWater deliveryRunningWater) {
        deliveryRunningWater.setCreateTime(new Date());
        return deliveryRunningWaterDao.insert(deliveryRunningWater);
    }

    @Override
    public Integer addDeliveryInsert(String appVersion, String apiVersion, Delivery delivery, String orderSource, String userId) {
        return addDeliveryInsert(appVersion, apiVersion, delivery, orderSource, userId, "");
    }

    @Override
    public Integer addDeliveryInsert(String appVersion, String apiVersion, Delivery delivery, String orderSource, String userId, String... spare) {
        StringBuffer sb = new StringBuffer();
        sb.append("快递公司：").append(delivery.getExpId()).append("，订单号：").append(delivery.getExpcode()).append("，手机号：").append(delivery.getUserPhone())
                .append("，取货码：").append(delivery.getShelfCode()).append("，快递公司：").append(delivery.getExpId());
        return this.insert(DeliveryRunningWater.builder()
                .operating(sb.toString())
                .deliveryId(delivery.getId())
                .storeId(delivery.getStoreId())
                .userId(userId)
                .appVersion(appVersion)
                .apiVersion(apiVersion)
                .operatingType(1)
                .orderSource(orderSource)
                .spare1(spare.length == 1 ? spare[0] : null)
                .spare2(spare.length == 2 ? spare[1] : null)
                .spare3(spare.length == 3 ? spare[2] : null)
                .build());
    }

    @Override
    public Integer updateDeliveryInsert(Delivery oldDelivery, Delivery newDelivery, String userId, String appVersion, String apiVersion) {
        Map<String, Map<String, Object>> stringMapMap = ClassCompareUtil.compareFields(oldDelivery, newDelivery);
        StringBuilder stringBuilder = new StringBuilder("");
        for (String key : stringMapMap.keySet()) {
            Map<String, Object> value = stringMapMap.get(key);
            Object newValue = value.get("newValue");
            Object oldValue = value.get("oldValue");
            stringBuilder.append(key).append("字段，原值：").append(oldValue).append("，新值：").append(newValue).append("；");
        }
        return updateDeliveryInsert(newDelivery, newDelivery.getStoreId(), userId, stringBuilder.toString(), appVersion, apiVersion);
    }

    @Override
    public Integer updateDeliveryInsert(Delivery newDelivery, Integer storeId, String userId, String updateContent, String appVersion, String apiVersion) {
        return this.insert(DeliveryRunningWater.builder()
                .operating(updateContent.length() > 500 ? updateContent.substring(0, 499) : updateContent)
                .deliveryId(newDelivery.getId())
                .storeId(storeId)
                .userId(userId)
                .appVersion(appVersion)
                .apiVersion(apiVersion)
                .operatingType(2)
                .build());
    }
}
