package com.parksong.services.delivery.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.parksong.basics.beans.BatchDataMsg;
import com.parksong.basics.exception.AskException;
import com.parksong.basics.exception.BusinessException;
import com.parksong.basics.filter.ApiFilter;
import com.parksong.basics.services.BasicsService;
import com.parksong.basics.services.RabbitService;
import com.parksong.basics.util.basic.DateUtil;
import com.parksong.basics.util.basic.StringsUtil;
import com.parksong.basics.util.configurations.RabbitConfig;
import com.parksong.basics.util.constants.*;
import com.parksong.basics.util.http.HttpUtil;
import com.parksong.basics.util.redis.CacheTool;
import com.parksong.beans.delivery.*;
import com.parksong.beans.express.ExpressType;
import com.parksong.beans.store.Store;
import com.parksong.beans.store.StoreShelf;
import com.parksong.beans.store.WhiteList;
import com.parksong.beans.user.User;
import com.parksong.dao.delivery.DeliverysDao;
import com.parksong.factory.infoplatform.BaseInfoPlatform;
import com.parksong.factory.infoplatform.InfoPlatformFactory;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.filter.delivery.StorageDeliveryFilter;
import com.parksong.services.delivery.*;
import com.parksong.services.express.ExpressConsumeService;
import com.parksong.services.sms.SmsService;
import com.parksong.services.store.*;
import com.parksong.util.ExcelUtil;
import com.parksong.util.RegexUtil;
import com.parksong.util.bacth.BatchDataHandleUtil;
import com.parksong.util.enums.DeliverySignReturnEnum;
import com.parksong.util.enums.ExpressStatusEnum;
import com.parksong.util.enums.SMSErrorCodeEnum;
import com.parksong.util.enums.SmsSendStatusEnum;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletResponse;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

/**
 * @ClassName DeliveryServiceImpl
 * @Description 业务表service
 * @Author kael
 * @Date 2019/9/6 0006 下午 14:27
 * @Version 1.0
 **/
@Service
@Slf4j
public class DeliverysServiceImpl extends BasicsService implements DeliverysService {

    @Autowired
    private DeliverysDao deliverysDao;

    @Autowired
    private RabbitService rabbitService;

    @Autowired
    private StoreService storeService;

    @Autowired
    private WhiteListService whiteListService;

    @Autowired
    private StoreShelfService storeShelfService;

    @Autowired
    private DeliveryStatusService deliveryStatusService;

    @Autowired
    private DeliveryScanBatchService deliveryScanBatchService;

    @Autowired
    private ConfigurationService configurationService;

    @Autowired
    private DeliveryRunningWaterService deliveryRunningWaterService;

    @Autowired
    private SmsService smsService;

    @Autowired
    @Lazy
    private DeliveryAsyncService deliveryAsyncService;

    @Autowired
    private PickupQueueService pickupQueueService;

    @Autowired
    private ExpressConsumeService expressConsumeService;

    @Autowired
    private RabbitConfig rabbitConfig;

    @Autowired
    private InfoPlatformFactory infoPlatformFactory;

    @Override
    public List<Delivery> querySignReturn(DeliveryFilter filter) {
        filter.sortInit();
        List<Delivery> deliveries = deliverysDao.selectList(filter);
        deliveries.forEach(delivery -> delivery.setReturnSignStatusName(DeliverySignReturnEnum.of(delivery.getReturnSignStatus())));
        return deliveries;
    }

    @Override
    public Long querySignReturnCount(DeliveryFilter filter) {
        return deliverysDao.selectCount(filter);
    }

    @Override
    public List<BatchDataMsg> signReturnByIds(DeliveryFilter filter) {
//        if(filter.getDeliveryIds().size() > DeliveryConstants.MAX_BATCH_NUM){
//            throw new BusinessException("批量接口最多每次传入"+DeliveryConstants.MAX_BATCH_NUM+"条");
//        }
        List<BatchDataMsg> batchErrorMsgs = new ArrayList<>();
        filter.getDeliveryIds().forEach(deliveryId -> {
            try {
                filter.setDeliveryId(deliveryId);
                Delivery delivery = deliverysDao.queryDeliveryByIdOrExpCode(filter);
                if (delivery == null) {
                    throw new BusinessException("订单未找到！");
                }
                String status = "";
                if (delivery.getReturnSignStatus() == DeliverySignReturnConstants.SIGNINGFILS) {
                    status = "out";
                } else if (delivery.getReturnSignStatus() == DeliverySignReturnConstants.REPLACE_SIGNFILS) {
                    status = "in";
                }
                if ("".equals(status)) {
                    throw new BusinessException("该订单已经是代签或者签收完成");
                }
                this.signBack(delivery.getExpId(), delivery.getExpcode(), delivery.getUserPhone(), delivery.getStoreId(), status, (delivery.getShelfId() == null ? "" : delivery.getShelfId() + ""), delivery.getShelfCode(), ("CaiNiao-B").equals(delivery.getPlatform()) ? "CaiNiao" : delivery.getPlatform());
                batchErrorMsgs.add(BatchDataHandleUtil.batchSuccess(deliveryId.toString()));
            } catch (BusinessException e) {
                batchErrorMsgs.add(BatchDataHandleUtil.batchError(deliveryId.toString(), e.getMessage()));
                e.printStackTrace();
            } catch (Exception e) {
                batchErrorMsgs.add(BatchDataHandleUtil.batchError(deliveryId.toString(), "操作失败"));
                e.printStackTrace();
            }
        });
        return batchErrorMsgs;
    }

    @Override
    public void signBack(Integer expId, String expCode, String userPhone, Integer storeId, String status, String shelfId, String shelfCode, String platform) {
        rabbitService.send(rabbitConfig.getExpressExchange(), rabbitConfig.getExpressKey(), JSON.toJSONString(SignReturnBean.builder()
                .expId(expId)
                .expCode(expCode)
                .telPhone(userPhone)
                .store(storeService.getStoreById(storeId))
                .actionType(status)
                .shelfId(shelfId)
                .shelfCode(shelfCode)
                .expressSignBean(storeService.getSignBackSwitch(expId, storeId))
                //判断是否开启调用居欢颜
                .jhySignBean(storeService.getSignBackSwitch(9999, storeId))
                .platform(platform)
                .build()));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<BatchDataMsg> batchStorageDelivery(DeliveryFilter filter, String appVersion, String apiVersion) {
//        if(filter.getDeliverys().size() > DeliveryConstants.MAX_BATCH_NUM){
//            throw new BusinessException("批量接口最多每次传入"+DeliveryConstants.MAX_BATCH_NUM+"条");
//        }
        List<BatchDataMsg> batchErrorMsgs = new ArrayList<>();
        CacheTool cacheTool = new CacheTool("inDelivery:count");
        Map<Integer, List<WhiteList>> map = whiteListService.getMapByStoreId(filter.getStoreId());
        Store store = storeService.getStoreById(filter.getStoreId());
        User user = filter.getUser();
        log.info("【入库门店】：{}，【入库用户】：{}", store.getId(), user.getUserName());
        filter.getDeliverys().stream().forEach(storageDeliveryFilter -> {
            try {
                this.storageDelivery(filter, appVersion, apiVersion, storageDeliveryFilter, cacheTool, map, store, user);
                batchErrorMsgs.add(BatchDataHandleUtil.batchSuccess(storageDeliveryFilter.getExpCode()));
            } catch (Exception e) {
                batchErrorMsgs.add(BatchDataHandleUtil.batchError(storageDeliveryFilter.getExpCode(), "操作失败"));
                e.printStackTrace();
                //手动回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            }
        });
        return batchErrorMsgs;
    }

    @Override
    public boolean storageDelivery(DeliveryFilter filter, String appVersion, String apiVersion) {
        CacheTool cacheTool = new CacheTool("inDelivery:count");
        Map<Integer, List<WhiteList>> map = whiteListService.getMapByStoreId(filter.getStoreId());
        Store store = storeService.getStoreById(filter.getStoreId());
        User user = filter.getUser();
        log.info("入库门店：" + store.getShortName() + "------->入库用户：" + user.getUserName());
        this.storageDelivery(filter, appVersion, apiVersion, filter.getDeliverys().get(0), cacheTool, map, store, user);
        return true;
    }

    @Override
    public List<BatchDataMsg> batchSignflag(DeliveryFilter filter, String appVersion, String apiVersion) {
        List<BatchDataMsg> batchDataMsgs = new ArrayList<>();
        User user = filter.getUser();
        Integer storeId = filter.getStoreId();
        List<String> search;
        boolean isExpCode = false;
        if (filter.getDeliveryIds() != null && filter.getDeliveryIds().size() > 0) {
            search = filter.getDeliveryIds().stream().map(x -> x + "").collect(Collectors.toList());
        }else{
            isExpCode = true;
            search = filter.getExpCodes();
        }
        final boolean finalIsExpCode = isExpCode;
        Map<String,Integer> pickup = new HashMap<>(16);
        StringBuilder msg = new StringBuilder("近一周内:");
        Map<String,String> userPhoneMap = new HashMap<>(16);
        search.forEach(idorExpCode -> {
            try {
                if(finalIsExpCode){
                    filter.setExpCode(idorExpCode);
                }else{
                    filter.setDeliveryId(Integer.parseInt(idorExpCode));
                }
                Delivery delivery = deliverysDao.queryDeliveryByIdOrExpCode(filter);
                if (null == delivery){
                    throw new BusinessException("订单不存在！");
                }
                if (finalIsExpCode){
                    userPhoneMap.put("取件客户电话号码",delivery.getUserPhone());
                }
                if (delivery.getStatus().compareTo(DeliveryStatusConstants.ORDER_STATUS_LEAVE_STORE) == 0||
                        delivery.getStatus().compareTo(DeliveryStatusConstants.ORDER_STATUS_COMPLETE) == 0) {
                    throw new BusinessException("订单已出库,请勿重复签收！");
                }
                if (!CollectionUtils.isEmpty(filter.getExpSignPhotos())){
                    delivery.setSignPhotoUrl(StringUtils.join(filter.getExpSignPhotos(), ","));
                }
                delivery.setStoreId(storeId);
                Delivery entity = new Delivery();
                BeanUtils.copyProperties(delivery, entity);
                entity.setStatus(DeliveryStatusConstants.ORDER_STATUS_COMPLETE);
                entity.setStatusTime(new Date());
                entity.setMoney(0);
                entity.setPayType(PayTypeConstants.PAY_TYPE_ONLINE);
                entity.setDeliveryUid(user.getUserId());
                // 1:自提签收， 2： 派送签收。如果当前状态时派送中，出库方式为派送签收否则为自提签收
                entity.setSignWay(delivery.getStatus().compareTo(DeliveryStatusConstants.ORDER_STATUS_DISPATCHING) == 0 ? 2 : 1);

                entity.setUploadDtm(new Date());
                entity.setSignStatus(1);
                Integer count = deliverysDao.update(entity);
                if (count.compareTo(0) > 0) {
                    //修改总表
                    deliveryAsyncService.updateDeliveryToday(entity);
                    // 快递公司消费记录
                    expressConsumeService.insertConsumeRecord(delivery, ExpressAccountConstants.DEDUCTION_NODE_COMPLETE);
                    //添加订单流水
                    deliveryRunningWaterService.updateDeliveryInsert(delivery, entity, user.getUserId(), appVersion, apiVersion);
                    this.signBack(delivery.getExpId(), delivery.getExpcode(), delivery.getUserPhone(), user.getStoreId(), "out", null, null, ("CaiNiao-B").equals(delivery.getPlatform()) ? "CaiNiao" : delivery.getPlatform());
                    // 发送微信签收消息
                    this.wxSignMsgRemind(entity);
                } else {
                    throw new BusinessException("出库失败，订单修改失败");
                }
                pickup.merge(delivery.getUserPhone(), 1, Integer::sum);
                batchDataMsgs.add(BatchDataHandleUtil.batchSuccess(idorExpCode));
            } catch (BusinessException e) {
                batchDataMsgs.add(BatchDataHandleUtil.batchError(idorExpCode, e.getMessage()));
                e.printStackTrace();
            } catch (Exception e) {
                batchDataMsgs.add(BatchDataHandleUtil.batchError(idorExpCode, "操作失败"));
                e.printStackTrace();
            }
        });
        //开启联想签收功能
        boolean isSignAssociation = filter.getIsSignAssociation() == null ? false : filter.getIsSignAssociation();
        if (isSignAssociation){
            String userPhone = userPhoneMap.get("取件客户电话号码");
                String msgStr = msgSignAssociation(DeliveryFilter.builder().storeId(storeId).userPhone(userPhone).build());
                if (msgStr != null){
                    msg.append(msgStr).append(";");
                }
            if (msg.length() > 5){
                batchDataMsgs.add(BatchDataHandleUtil.batchSuccess("取件签收提醒消息", userPhone,msg));
            } else {
                batchDataMsgs.add(BatchDataHandleUtil.batchSuccess("取件签收提醒消息"));
            }
        }
        pickupQueueService.pickupSign(pickup);
        return batchDataMsgs;
    }

    private String msgSignAssociation(DeliveryFilter deliveryFilter ) {
        deliveryFilter.setSearchExpTime(LocalDate.now().plusDays(-6).toString() + " 0:00:00");
        int expCount = deliverysDao.getUnsignExpCount(deliveryFilter);
        return expCount > 0 ? deliveryFilter.getUserPhone() + "客户还有" + expCount + "条快递未出库" : null;
    }

    @Override
    public List<Delivery> queryDelivery(DeliveryFilter filter) throws ParseException {
        filter.sortInit();
        // 查询前的filter数据处理
        this.queryDeliveryFilterHandle(filter);
        //订单查询数据处理
        return getDeliveries(deliverysDao.selectList(filter),filter.getStoreId());
    }

    /**
     * 订单查询数据处理
     * @param deliveries
     * @param storeId
     * @return
     */
    private List<Delivery> getDeliveries(List<Delivery> deliveries,Integer storeId) {
        // 根据店铺id查询当前店铺的白名单客户
        Map map = whiteListService.getPhoneMapByStoreId(storeId);
        // 订单处理，返回对应参数
        return this.returnDeliveryHandle(deliveries, map);
    }

    @Override
    public Long queryDeliveryCount(DeliveryFilter filter) throws ParseException {
        // 查询前的filter数据处理
        this.queryDeliveryFilterHandle(filter);
        return deliverysDao.selectCount(filter);
    }

    @Override
    public DeliveryStatistics queryDStatistics(DeliveryFilter filter) throws ParseException {
        // 未通知/通知管理 接口
        if (!"2".equals(filter.getPageType())) {
            this.queryDeliveryFilterHandle(filter);
            filter.setSmsStatus(-1);
            long noTice = deliverysDao.selectCount(filter);
            filter.setSmsStatus(1);
            long scuessTice = deliverysDao.selectCount(filter);
            filter.setSmsStatus(2);
            long fileTice = deliverysDao.selectCount(filter);

            return DeliveryStatistics.builder()
                    .noTice(noTice)
                    .scuessTice(scuessTice)
                    .fileTice(fileTice)
                    .sendTice(scuessTice + fileTice).build();
        } else {
            throw new NullPointerException();
        }
    }

    @Override
    // 在不确定是否能明确代发短信的时候？ 以电话号码参数判断是否能代发短信
    public GetInfoByExpCodeBean getInfoByExpCode(DeliveryFilter filter, String appVersion, String apiVersion) throws Exception {
        GetInfoByExpCodeBean getInfoByExpCodeBean = null;
        /*
            获取手机号/代发顺序规则，依次为 快递公司，菜鸟，拼多多
         */
        String[] platforms = new String[]{String.valueOf(filter.getExpId()), "CaiNiao", "PDD"};
        for (int i = 0; i < platforms.length; i++) {
            BaseInfoPlatform platform = infoPlatformFactory.getPlatform(platforms[i]);
            if (platform == null) {
                continue;
            }
            getInfoByExpCodeBean = platform.getInfoByExpCode(filter);
            if (null != getInfoByExpCodeBean && !Strings.isBlank(getInfoByExpCodeBean.getTelPhone())) {
                break;
            }
        }
        return getInfoByExpCodeBean;
    }

    @Override
    public List<GuoanFtpData> queryDeliveryByGuoan(DeliveryFilter filter) {
        return deliverysDao.queryDeliveryByGuoan(filter);
    }

    @Override
    public Integer updateDeliverySmsStatus(DeliveryFilter filter) {
        return deliverysDao.updateDeliverySmsStatus(filter);
    }

    @Override
    public Map<Integer, Delivery> queryMapByDeliveryIds(List<Integer> deliveryIds, Integer storeId) {
        DeliveryFilter filter = new DeliveryFilter();
        filter.setDeliveryIds(deliveryIds);
        filter.setStoreId(storeId);
        return deliverysDao.queryMapByDeliveryIds(filter);
    }

    @Override
    public Integer insertTotalDelivery(Delivery delivery) {
        //插入总表
        return deliverysDao.insertTotalDelivery(delivery);
    }


    @Override
    public Integer updateDeliveryToday(Delivery delivery) {
        //插入总表
        return deliverysDao.updateDeliveryToday(delivery);
    }

    @Override
    public Integer saveOrUpdateStatToday(Delivery delivery) {
        BaseStatToday statToday = new BaseStatToday();

        statToday.setStoreId(delivery.getStoreId());
        statToday.setExpressId(delivery.getExpId());
        statToday.setStatType(delivery.getStatus());
        statToday.setNums(1);
        statToday.setPrice(delivery.getChargeMoney());
        statToday.setRealMoney(delivery.getMoney());
        statToday.setCreateTimeNum(0);
        statToday.setStatusTimeNum(0);
        statToday.setUnComplete(0);
        statToday.setStatTime(new Date());
        //1 昨日 2 当日
        statToday.setSource(2);

        if (StoreDeliveryTypeConstants.ORDER_SEND_TYPE_ZT.equals(delivery.getOdeliveryType())) {
            //自提
            statToday.setOutType(1);
        } else if ("PS".equals(delivery.getOdeliveryType())) {
            //派送
            statToday.setOutType(2);
        }

        if (Strings.isEmpty(delivery.getUserPhone())) {
            statToday.setUnComplete(1);
        }

        return deliverysDao.saveOrUpdateStatToday(statToday);
    }

    @Override
    public Integer pushTodayToYesterday(BaseStatToday baseStatToday) {
        return deliverysDao.pushTodayToYesterday(baseStatToday);
    }

    @Override
    public List<Delivery> getDeliveryByPhones(DeliveryFilter filter){
        return deliverysDao.getDeliveryByPhones(filter);
    }

    @Override
    public Integer queryIdFromMasterDelivery(Delivery delivery) {
        return deliverysDao.queryIdFromMasterDelivery(delivery);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean modify(DeliveryFilter data, String appVersion, String apiVersion) {
        if (null == data.getDeliveryId() || null == data.getStoreId()) {
            throw new BusinessException("不合法的参数");
        }
        /*if (!Strings.isBlank(data.getShelfCode()) && data.getShelfCode().length() > 9) {
            throw new BusinessException("取件码不能大于9位");
        }*/
        if(Strings.isNotBlank(data.getUserPhone())){
            boolean userPhoneValid =  RegexUtil.isValidPhoneNumberCN(data.getUserPhone())
                    || Constants.DEFAULT_MOBILE_KB.equals(data.getUserPhone());
            if (!userPhoneValid) {
                throw new BusinessException("手机号码不符合规则");
            }
        }

        DeliveryFilter filter = new DeliveryFilter();
        filter.setStoreId(data.getStoreId());
        filter.setDeliveryId(data.getDeliveryId());
        Delivery delivery = deliverysDao.queryDeliveryByIdOrExpCode(filter);

        if (null == delivery) {
            throw new BusinessException("订单不存在");
        }
        if (delivery.getStatus().compareTo(DeliveryStatusConstants.ORDER_STATUS_LEAVE_STORE) == 0||
                delivery.getStatus().compareTo(DeliveryStatusConstants.ORDER_STATUS_COMPLETE) == 0) {
            throw new BusinessException("订单不在库");
        }

        Delivery entity = new Delivery();
//        boolean isModify = false;
        Integer masterId = null;
        StringBuilder stringBuilder = new StringBuilder();
        // 快递公司发生变更
        if (null != data.getExpId() && data.getExpId().compareTo(delivery.getExpId()) != 0) {
            entity.setExpId(data.getExpId());
//            isModify = true;
            masterId = queryIdFromMasterDelivery(delivery);
            stringBuilder.append("expId").append("字段，原值：").append(delivery.getExpId()).append("，新值：").append(data.getExpId()).append("；");
        }
        // 面单号发生变更
        if (!Strings.isBlank(data.getExpCode()) && !data.getExpCode().equals(delivery.getExpcode())) {
            entity.setExpcode(data.getExpCode());
            int length;
            /*String vagueExpCode = (length=data.getExpCode().length())>4?data.getExpCode().substring(length-4):data.getExpCode();
            entity.setVagueExpCode(vagueExpCode);*/
//            isModify = true;
            stringBuilder.append("expCode").append("字段，原值：").append(delivery.getExpcode()).append("，新值：").append(data.getExpCode()).append("；");
//            stringBuilder.append("vagueExpCode").append("字段").append("，新值：").append(vagueExpCode).append("；");
        }
        // 取件码发生变更
        if (!Strings.isBlank(data.getShelfCode()) && !data.getShelfCode().equals(delivery.getShelfCode())) {
            entity.setShelfCode(data.getShelfCode());
//          isModify = true;
            stringBuilder.append("shelfCode").append("字段，原值：").append(delivery.getShelfCode()).append("，新值：").append(data.getShelfCode()).append("；");
        }
        // 手机号发生变更
        if (!Strings.isBlank(data.getUserPhone()) && !data.getUserPhone().equals(delivery.getUserPhone())) {
            entity.setUserPhone(data.getUserPhone());
            int length;
            /*String vagueUserPhone = (length = data.getUserPhone().length()) > 4 ? data.getUserPhone().substring(length - 4) : data.getUserPhone();
            entity.setVagueUserPhone(vagueUserPhone);*/
//            isModify = true;
            stringBuilder.append("userPhone").append("字段，原值：").append(delivery.getUserPhone()).append("，新值：").append(data.getUserPhone()).append("；");
//            stringBuilder.append("vagueUserPhone").append("字段").append("，新值：").append(vagueUserPhone).append("；");
        }

//        if (!isModify) {
//            return true;
//        }

        // 延时发送短信入库时,这条快递又被编辑了,将入库的那条快递短信的key删除,避免发送两条消息
        // 入库即发短信时编辑快递信息就将key给删除,点击发送通知时可以立即发送短信
        CacheTool cacheTool = CacheTool.initSMSFlag();
        String smsKey = delivery.getExpId() + "_" + delivery.getExpcode() + "_" + delivery.getUserPhone() + "_" + delivery.getStoreId();
        cacheTool.del(smsKey);

        // 快递内容编辑后,将smsStatus修改为-1,点击发送通知时发送短信
        entity.setSmsStatus(-1);
        entity.setStoreId(data.getStoreId());
        entity.setId(delivery.getId());

        // 更新业务表数据
        deliverysDao.update(entity);
        // 更新总表数据
        deliveryAsyncService.updateMasterDelivery(entity, delivery);
        // 记录流水
        deliveryRunningWaterService.updateDeliveryInsert(entity, data.getStoreId(), data.getUser().getUserId(),
                stringBuilder.toString(), appVersion, apiVersion);
        // 快递公司发生变化，插入新的快递公司消费记录
        if (null != entity.getExpId() && masterId != null) {
            Delivery newDelivery = new Delivery();
            BeanUtils.copyProperties(entity, newDelivery);
            if(Strings.isBlank(newDelivery.getExpcode())){
                newDelivery.setExpcode(delivery.getExpcode());
            }
            // 修改后的Delivery内容，id需要是总表的ID
            newDelivery.setId(masterId);
            expressConsumeService.expressChanged(delivery.getExpId(), newDelivery);

            /*
             * 在修改后的快递公司进行签收回传，默认进行入库签收回传
             * 无论当前状态如何都需要重新进行入库回传
             * 需要限制已出库包裹的不能修改信息
             */
            String status = "in";
//            if (delivery.getReturnSignStatus() == DeliverySignReturnConstants.SIGNINGFILS) {
//                status = "out";
//            } else if (delivery.getReturnSignStatus() == DeliverySignReturnConstants.REPLACE_SIGNFILS) {
//                status = "in";
//            }
            this.signBack(entity.getExpId(),
                    Optional.ofNullable(entity.getExpcode()).orElse(delivery.getExpcode()),
                    Optional.ofNullable(entity.getUserPhone()).orElse(delivery.getUserPhone()),
                    Optional.ofNullable(entity.getStoreId()).orElse(delivery.getStoreId()),
                    status,
                    (delivery.getShelfId() == null ? "" : delivery.getShelfId() + ""),
                    Optional.ofNullable(entity.getShelfCode()).orElse(delivery.getShelfCode()),
                    ("CaiNiao-B").equals(delivery.getPlatform()) ? "CaiNiao" : delivery.getPlatform());
        }
        return true;
    }

    @Override
    public Integer updateMasterDelivery(Delivery entity, Delivery param) {
        return deliverysDao.updateMasterDelivery(entity, param);
    }

    @Override
    public ResponsePage queryDeliveryForPage(DeliveryFilter filter) throws ParseException {
        ResponsePage responsePage = new ResponsePage();
        // 查询前的filter数据处理
        this.queryDeliveryFilterHandle(filter);
        responsePage.setList(getDeliveries(deliverysDao.selectList(filter),filter.getStoreId()));
        responsePage.setTotal(deliverysDao.selectCount(filter));
        return responsePage;
    }

    @Override
    public ExpressType getExpressByExpCode(ApiFilter<DeliveryFilter> filterApiFilter, String version) {
        return infoPlatformFactory.getPlatform("187").getExpressByExpCode(filterApiFilter.getData().getExpCode());
    }

    @Override
    public DeliveryInOutBoundDto queryInOutBound(ApiFilter<DeliveryFilter> filter) {
        DeliveryFilter data = filter.getData();
        Delivery queryParam = new Delivery();
        queryParam.setExpcode(Objects.requireNonNull(data.getExpCode()));
        String phone = Objects.requireNonNull(data.getUserPhone());
        queryParam.setUserPhone(phone.replaceAll("\\*","%"));
        queryParam.setExpId(Objects.requireNonNull(data.getExpId()));
        List<Delivery> list = deliverysDao.queryMasterDelivery(queryParam);
        if (list != null && !list.isEmpty()) {
            Integer storeId = list.get(0).getStoreId();
            data.setStoreId(storeId);
            data.setUserPhone(list.get(0).getUserPhone());
            // 获取运单信息
            List<Delivery> deliveries = deliverysDao.selectList(data);
            if (deliveries != null && !deliveries.isEmpty()) {
                Delivery delivery = deliveries.get(0);
                // 获取货架信息
                StoreShelf storeShelf = storeShelfService.getById(deliveries.get(0).getShelfId());
                // 获取门店信息
                Store store = storeService.getStoreById(storeId);
                DeliveryInOutBoundDto inOutBoundDto = new DeliveryInOutBoundDto();
                BeanUtils.copyProperties(store, inOutBoundDto);
                BeanUtils.copyProperties(delivery, inOutBoundDto);
                inOutBoundDto.setShelfCode(storeShelf.getShelfName() + "-" + deliveries.get(0).getShelfCode());
                return inOutBoundDto;
            }
        }
        return null;
    }

    @Override
    public StatisticsOutboundRate outboundRate(DeliveryFilter filter) {
        // 创建一个数值格式化对象
        NumberFormat numberFormat = NumberFormat.getInstance();
        // 设置精确到小数点后2位
        numberFormat.setMaximumFractionDigits(2);
        List<StatisticsOutboundRate> queryData = getQueryData(filter.getStoreId(), filter.getBeginTime(), filter.getEndTime(), filter.getExpId(), numberFormat);
        StatisticsOutboundRate totalData = getTotalData(queryData, numberFormat);
        return totalData;
    }

    @Override
    public void export(HttpServletResponse response, DeliveryFilter filter) {
        List<StatisticsOutboundRate> queryData = getQueryData(filter.getStoreId(), filter.getBeginTime(), filter.getEndTime(), null, null);
        ExcelUtil.exportExpressData(response, queryData);
    }

    @Override
    public List<PhoneLibDto> getConsigneeInfoByPhone(DeliveryFilter filter) {
        List<PhoneLibDto> phoneLibDtoList = deliverysDao.getConsigneeInfoByPhone(filter);
        if (StringUtils.isNotBlank(filter.getConsigneePhoneEnd())){
            phoneLibDtoList.stream().forEach(phoneLibDto -> phoneLibDto.setNamePhone(phoneLibDto.getConsingeeName() + "--" + phoneLibDto.getConsingeePhone()));
        }
        return phoneLibDtoList;
    }

    @Override
    public Delivery storageDeliveryNew(DeliveryFilter filter, String appVersion, String apiVersion) {
        Delivery delivery = new Delivery();
        CacheTool cacheTool = new CacheTool("inDelivery:count");
        Map<Integer, List<WhiteList>> map = whiteListService.getMapByStoreId(filter.getStoreId());
        Store store = storeService.getStoreById(filter.getStoreId());
        User user = filter.getUser();
        log.info("入库门店：" + store.getShortName() + "------->入库用户：" + user.getUserName());
        StorageDeliveryFilter storageDeliveryFilter = filter.getDeliverys().get(0);
        //处理取件码
        dealWithShelfCode(filter.getStoreId(), storageDeliveryFilter);
        this.storageDelivery(filter, appVersion, apiVersion, storageDeliveryFilter, cacheTool, map, store, user);
        delivery.setShelfCode(storageDeliveryFilter.getShelfCode());
        return delivery;
    }

    @Override
    public Delivery expressDetailInfo(DeliveryFilter filter) {
        Delivery delivery = deliverysDao.expressDetailInfo(filter.getDeliveryId(), filter.getStoreId());
        if (delivery.getStatus() == 126 || delivery.getStatus() == 246){
            //相隔时间
            Duration duration = Duration.between(delivery.getCreateTime().toInstant().atZone(ZoneId.systemDefault()).toLocalDate(), delivery.getStatusTime().toInstant().atZone(ZoneId.systemDefault()).toLocalDate());
            delivery.setWarehousingTime(duration.toDays());
        }else {
            delivery.setStatusTime(null);
            delivery.setWarehousingTime(0L);
        }
        delivery.setStatusName(ExpressStatusEnum.valueOf(delivery.getStatus()).getValue());
        delivery.setSmsStatusName(SmsSendStatusEnum.valueOf(delivery.getSmsStatus()).getValue());
        if (StringUtils.isNotBlank(delivery.getSignPhotoUrl())){
            delivery.setSignPhotoUrlList(Arrays.asList(delivery.getSignPhotoUrl().split(",")));
        }

        List<ExpressOperation> expressOperationList = deliverysDao.selectExpressOperationRecord(filter.getStoreId(), delivery.getExpcode());
        for (ExpressOperation expressOperation : expressOperationList) {
            String createTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(expressOperation.getCreateTime());
            DeliveryFilter build = DeliveryFilter.builder()
                    .storeId(expressOperation.getStoreId())
                    .expCode(expressOperation.getExpcode())
                    .build();
            Delivery delivery1 = deliverysDao.queryDeliveryByIdOrExpCode(build);
            if (expressOperation.getStatus() == 0) {
                expressOperation.setTitle(ExpressStatusEnum.valueOf(expressOperation.getStatus()).getValue() + SmsSendStatusEnum.valueOf(delivery1.getSmsStatus()).getValue());
                expressOperation.setOperationContent(createTime + " 操作人:" + delivery1.getDeliveryUname());
            } else {
                expressOperation.setTitle(ExpressStatusEnum.valueOf(expressOperation.getStatus()).getValue() + "成功");
                if (expressOperation.getStatus() == 122){
                    expressOperation.setOperationContent(createTime + " 操作人:" + delivery1.getDeliveryUname());
                }else {
                    expressOperation.setOperationContent(createTime + " 操作人:" + delivery1.getDeliveryUid());
                }
            }

        }
        delivery.setExpressOperationList(expressOperationList);
        return delivery;
    }

    @Override
    public boolean addSenderInfo(DeliveryFilter filter) {
        if (StringUtils.isBlank(filter.getSenderPhone())){
            throw new BusinessException("寄件人电话号码不能为空");
        }
        deliverysDao.addSenderInfo(filter);
        return true;
    }

    private void dealWithShelfCode(Integer storeId, StorageDeliveryFilter storageDeliveryFilter) {
        CacheTool shelfcodeCacheTool = CacheTool.initShelfcodeCacheTool();
        String day = String.format("%02d", LocalDate.now().getDayOfMonth());
        String redisKey = storeId + ":" + day + ":" + storageDeliveryFilter.getShelfId();
        String number = String.format("%03d", shelfcodeCacheTool.incrBy(redisKey, 1).intValue());
        StoreShelf storeShelf = storeShelfService.getById(storageDeliveryFilter.getShelfId());
        //取件码  货架号 + 当天日期 + 三位自增长
        storageDeliveryFilter.setShelfCode(storeShelf.getShelfName() + "-" + day + "-" + number);
        //是否存在相同取货码+在库的快递
        if (deliverysDao.selectDelivery(storeId, storageDeliveryFilter.getShelfCode()) > 0){
            throw new BusinessException("库中存在相同取件码,请及时处理：" + storageDeliveryFilter.getShelfCode());
        }

    }

    private StatisticsOutboundRate getTotalData(List<StatisticsOutboundRate> queryData, NumberFormat numberFormat) {
        StatisticsOutboundRate totalData = StatisticsOutboundRate.builder()
                .outboundNum(0)
                .warehousing(0)
                .waitOutboundNum(0)
                .sendBackNum(0)
                .outboundRate("0%")
                .detailInfo(queryData)
                .build();
        if (org.apache.commons.collections.CollectionUtils.isNotEmpty(queryData)){
            queryData.stream().forEach(data -> {
                totalData.setWarehousing(totalData.getWarehousing() + data.getWarehousing());
                totalData.setOutboundNum(totalData.getOutboundNum() + data.getOutboundNum());
            });
            totalData.setWaitOutboundNum(totalData.getWarehousing() - totalData.getOutboundNum());
            if(totalData.getWarehousing() == 0){
                totalData.setOutboundRate("0%");
            }else {
                totalData.setOutboundRate(numberFormat.format((float) totalData.getOutboundNum() / (float) totalData.getWarehousing() * 100) + "%");
            }

        }
        return totalData;
    }

    private List<StatisticsOutboundRate> getQueryData(Integer storeId, String beginTime, String endTime, Integer expId, NumberFormat numberFormat) {
        List<StatisticsOutboundRate> queryData = new ArrayList<>();
        List<String> timeList = deliverysDao.historyTime(storeId, beginTime, endTime);
        timeList.stream().forEach(time -> {
            queryData.add(getTodayData(storeId, time, expId, numberFormat));
        });
        return queryData;
    }

    private StatisticsOutboundRate getTodayData(Integer storeId, String todayTime, Integer expId, NumberFormat numberFormat) {
        StatisticsOutboundRate todayData = StatisticsOutboundRate.builder()
                .time(todayTime)
                .outboundNum(0)
                .warehousing(0)
                .waitOutboundNum(0)
                .sendBackNum(0)
                .outboundRate("0%")
                .detailInfo(new ArrayList<>())
                .build();
        List<StatisticsOutboundRate> statisticsOutboundRates = deliverysDao.todayData(storeId, todayTime, expId);
        if (org.apache.commons.collections.CollectionUtils.isNotEmpty(statisticsOutboundRates)){
            statisticsOutboundRates.stream().forEach(statisticsOutboundRate -> {
                statisticsOutboundRate.setWaitOutboundNum(statisticsOutboundRate.getWarehousing() - statisticsOutboundRate.getOutboundNum());
                if (null !=numberFormat){
                    statisticsOutboundRate.setOutboundRate(numberFormat.format((float) statisticsOutboundRate.getOutboundNum() / (float) statisticsOutboundRate.getWarehousing() * 100) + "%");
                }
                todayData.setWarehousing(todayData.getWarehousing() + statisticsOutboundRate.getWarehousing());
                todayData.setOutboundNum(todayData.getOutboundNum() + statisticsOutboundRate.getOutboundNum());
                todayData.getDetailInfo().add(statisticsOutboundRate);
            });
            todayData.setWaitOutboundNum(todayData.getWarehousing() - todayData.getOutboundNum());
            if (null != numberFormat){
                if(todayData.getWarehousing() == 0){
                    todayData.setOutboundRate("0%");
                }else {
                    todayData.setOutboundRate(numberFormat.format((float) todayData.getOutboundNum() / (float) todayData.getWarehousing() * 100) + "%");
                }
            }
        }
        return todayData;
    }

    /**
     * 单个订单入库
     * @param filter
     * @param appVersion
     * @param apiVersion
     * @param storageDeliveryFilter
     * @param cacheTool
     * @param map
     * @param store
     * @param user
     */
    private void storageDelivery(DeliveryFilter filter, String appVersion, String apiVersion, StorageDeliveryFilter storageDeliveryFilter, CacheTool cacheTool, Map<Integer, List<WhiteList>> map, Store store, User user) {
        log.info("【快递单号】：{}，【快递公司】：{}，开始入库", storageDeliveryFilter.getExpCode(), storageDeliveryFilter.getExpId());
        boolean whiteUser = false;
        //去redis中检查是否存在相同记录
        String stringKey = cacheTool.getStringKey(storageDeliveryFilter.getExpId() + "_" + storageDeliveryFilter.getExpCode() + "_" + storageDeliveryFilter.getShelfId() + "_" + storageDeliveryFilter.getShelfCode());
        if (!Strings.isBlank(stringKey)) {
            throw new BusinessException("面单号重复");
        }
        if(!expressConsumeService.checkExpressBalance(filter.getStoreId(), storageDeliveryFilter.getExpId())){
            String express = CacheTool.initExpress().getString(storageDeliveryFilter.getExpId().toString());
            if (StringUtils.isNotBlank(express)){
                ExpressType expressCompany = JSONObject.parseObject(express, ExpressType.class);
                throw new BusinessException(expressCompany.getExpName() + "快递余额不足，请联系管理员充值");
            }else{
                throw new BusinessException("快递余额不足，请联系管理员充值");
            }
        }
        if (storageDeliveryFilter.getTelPhone() != null){
            //白名单用户
            List<WhiteList> whiteLists = map.get(filter.getStoreId());
            if (whiteLists != null && whiteLists.size() > 0) {
                whiteLists = whiteLists.stream().filter(whiteList -> whiteList.getUsermobile().equals(storageDeliveryFilter.getTelPhone())).collect(Collectors.toList());
                if (whiteLists.size() > 0) {
                    whiteUser = true;
                }
                if (whiteUser && !"yes".equals(storageDeliveryFilter.getWhiteIsInbound())) {
                    throw new AskException("白名单用户是否需要入库");
                }
            }
        }
        StoreShelf storeShelf = storeShelfService.getById(storageDeliveryFilter.getShelfId());
        if (storeShelf == null) {
            throw new BusinessException("货架不存在");
        }
        //入库
        //"ZI";
        String odeliveryType = StoreDeliveryTypeConstants.ORDER_SEND_TYPE_ZT;
        if (store.getIsP2p() == 1) {
            //"P2P";
            odeliveryType = StoreDeliveryTypeConstants.ORDER_SEND_TYPE_P2P;
        }
        /**《顺丰和当当的订单, 尝试从未到货表自动完善》该段代码未迁移**/
        Delivery delivery = null;
        try{
            delivery = insertDelivery(store, storageDeliveryFilter, whiteUser, odeliveryType, user.getUserId());
        }catch (Exception e){
            throw new BusinessException("入库失败");
        }

        //下面代码执行顺序中，手机号同步和入库签收回传调用必须在最后面，防止事务回滚却发送了签收回传和手机号同步mq
        if (delivery != null && delivery.getId() != null) {
            //新增订单状态
            this.insertDeliveryStatus(store.getId(), user.getUserName(), delivery.getId());
            /**《短信发送方式为自动发送并且为免费模式时，发送入库短信》该段注释代码未迁移**/
            this.insertDeliveryScanBatch(store.getId(), storageDeliveryFilter);
            cacheTool.setStringandDatakey(storageDeliveryFilter.getExpId() + "_" + storageDeliveryFilter.getExpCode() + "_" + storageDeliveryFilter.getShelfId() + "_" + storageDeliveryFilter.getShelfCode(), "0", 3600);
            if (StringUtils.isNotBlank(storageDeliveryFilter.getTelPhone())){
                //短信自动发送开关
                this.smsAutomaticSend(user, delivery,whiteUser);
                //手机号同步
                this.phoneLibPush(filter, store.getId(), storageDeliveryFilter.getTelPhone());
            }
            //添加订单流水记录
            deliveryRunningWaterService.addDeliveryInsert(appVersion, apiVersion, delivery, HttpUtil.getDeviceType(), user.getUserId(),HttpUtil.getHeader("imei"));
            //入库签收回传调用
            this.signBack(storageDeliveryFilter.getExpId(), storageDeliveryFilter.getExpCode(), storageDeliveryFilter.getTelPhone(), filter.getStoreId(), "in", storageDeliveryFilter.getShelfId() + "", storageDeliveryFilter.getShelfCode(), ("CaiNiao-B").equals(storageDeliveryFilter.getPlatform()) ? "CaiNiao" : storageDeliveryFilter.getPlatform());
        } else {
            throw new BusinessException("入库失败，新增订单数据为空");
        }
    }

    /**
     * 查询前的filter数据处理
     *
     * @param filter
     * @throws ParseException
     */
    private void queryDeliveryFilterHandle(DeliveryFilter filter) throws ParseException {
        filter.sortInit();
        // 获取开始和结束时间
        this.getBeginAndEndTime(filter);

        if (filter.getSmsStatus().compareTo(-2) == 0) {
            filter.setSmsStatus(3);
        }
        // 通过pageType获取查询状态条件
        this.getQueryStatus(filter);
        // 通过input获取查询输入框条件
        this.getQueryInput(filter);
        // 判断是否是未通知接口，不显示白名单用户
        /*if ( "-1".equals(filter.getPageType())) {
            filter.setWhiteUser(false);
        }*/
    }

    /**
     * 订单处理，返回对应参数
     *
     * @param list 查询出来的订单集合
     * @param map
     * @return 处理后的订单集合
     */
    private List<Delivery> returnDeliveryHandle(List<Delivery> list, Map map) {
        List<Delivery> result = list.stream().sorted((o1, o2) -> {
            if (o2.getCreateTime().getTime() - o1.getCreateTime().getTime() > 0) {
                return 1;
            } else if (o2.getCreateTime().getTime() - o1.getCreateTime().getTime() < 0) {
                return -1;
            }
            return 0;
        }).map(item -> {
            Delivery jdo = JSON.parseObject(JSON.toJSONString(item), Delivery.class);
            //判断是否有白名单用户
            if (map.get(item.getUserPhone()) != null) {
                jdo.setWhiteUser(true);
                jdo.setIsSendSMS(1);
            } else {
                jdo.setWhiteUser(false);
                if (item.getStatus() == DeliveryStatusConstants.ORDER_STATUS_STORED || item.getStatus() == DeliveryStatusConstants.ORDER_STATUS_REJECT_RETENTION) {
                    jdo.setIsSendSMS(0);
                } else {
                    jdo.setIsSendSMS(1);
                }
            }
            jdo.setSmsStatus(null == item.getSmsStatus() ? -1 : item.getSmsStatus());
            if (null != item.getResultInfo() && null != item.getSmsStatus() && 2 == item.getSmsStatus()) {
                jdo.setResultInfo(SMSErrorCodeEnum.of(item.getResultInfo()));
            } else {
                jdo.setResultInfo("");
            }
            if (null != item.getReturnSignStatus()) {
                jdo.setReturnSignStatusName(DeliverySignReturnEnum.of(item.getReturnSignStatus()));
            }
            if (null == item.getSendSuccessType()) {
                jdo.setSendSuccessType(0);
            } else {
                jdo.setSendSuccessType(item.getSendSuccessType());
            }
            return jdo;
        }).collect(Collectors.toList());
        return result;
    }

    /**
     * 通过input获取查询输入框条件
     *
     * @param filter
     */
    private void getQueryInput(DeliveryFilter filter) {
        if (!Strings.isBlank(filter.getInput())) {
            int len = filter.getInput().length();
            switch (filter.getQueryType()) {
                // 根据面单号查询
                case 1:
                    if (len > 7) {
                        filter.setExpCode(filter.getInput());
                    } else if (len > 4) {
                        filter.setRevexpCode(filter.getInput().substring(len - 4));
                    } else {
                        filter.setRevexpCode(filter.getInput());
                    }
                    break;
                // 根据用户手机号查询
                case 2:
                    if (len == 11) {
                        filter.setUserPhone(filter.getInput());
                    } else if (len > 4) {
                        filter.setRevuserPhone(filter.getInput().substring(len - 4));
                    } else {
                        filter.setRevuserPhone(filter.getInput());
                    }
                    break;
                // 根据取件码查询
                case 3:
                    filter.setShelfCode(filter.getInput());
                    break;
                case 4:
                    filter.setUserName(filter.getInput());
                default:
            }
        }
    }

    /**
     * 通过pageType获取查询状态条件
     *
     * @param filter
     */
    private void getQueryStatus(DeliveryFilter filter) {
        //状态和类型没处理
        List<Integer> statusList = new ArrayList<>();
        //如果传了订单状态那么就以传入的为主，如果没传那么就通过pageType字段判断
        if (filter.getStatusType() != null) {
            statusList.add(filter.getStatusType());
        } else {
            if(Strings.isBlank(filter.getPageType())){
                filter.setPageType("1");
            }
            switch (filter.getPageType()){
                // 未通知
                case "-1"://未通知页面
                    //只查在库单子
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_STORED);
                    //滞留
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_REJECT_RETENTION);
                    break;
                case "1"://其他页面
                    //只查在库单子
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_STORED);
                    //滞留
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_DISPATCHING);
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_REJECT_RETENTION);
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_COMPLETE);
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_LEAVE_STORE);
                    break;
                case "3"://未签收页面
                    //滞留
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_REJECT_RETENTION);
                    //派送中
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_DISPATCHING);
                    //只查在库单子
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_STORED);
                    break;
                case "5":
                    // 滞留件画面
                    // 滞留
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_REJECT_RETENTION);
                    // 派送中
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_DISPATCHING);
                    // 在库
                    statusList.add(DeliveryStatusConstants.ORDER_STATUS_STORED);
                    break;
            }
        }
        filter.setStatusList(statusList);
    }

    /**
     * 获取开始和结束时间
     *
     * @param filter
     * @throws ParseException
     */
    private void getBeginAndEndTime(DeliveryFilter filter) throws ParseException {
        // 滞留件查询
        if ("5".equals(filter.getPageType())) {
            // 获取开始和结束时间
            long minusDays = 0;
            switch (Optional.ofNullable(filter.getTimeConditions()).orElse("")) {
                case "DAY_THREE":
                    minusDays = 3;
                    break;
                case "WEEK_ONE":
                    minusDays = 7;
                    break;
                default:
                    minusDays = 3;
                    break;
            }
            filter.setBeginTime(null);
            filter.setEndTime(LocalDate.now().minusDays(minusDays).format(DateTimeFormatter.ofPattern("yyyy-MM-dd 00:00:00")));
            return;
        }
        String pattern = "yyyy-MM-dd";
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        Date date = new Date();
        if (null == filter.getBeginTime() || "".equals(filter.getBeginTime())) {
            filter.setBeginTime(sdf.format(date));
        }
        filter.setBeginTime(filter.getBeginTime() + " 00:00:00");

        if (null == filter.getEndTime() || "".equals(filter.getEndTime())) {
            filter.setEndTime(sdf.format(date));
        }
        filter.setEndTime(DateUtil.addDay(filter.getEndTime(), pattern, 1) + " 00:00:00");
    }

    /**
     * 微信签收发送消息
     *
     * @param delivery
     */
    private void wxSignMsgRemind(Delivery delivery) {
        Map<String, Object> param = new HashMap<>(2);
        param.put("storeId", delivery.getStoreId());
        param.put("delivery", delivery);
        //发送mq
        rabbitService.send(rabbitConfig.getWechatExahcange(), rabbitConfig.getWechatKey(), JSON.toJSONString(param));
    }

    /**
     * 自动发送短信
     *
     * @param user
     * @param delivery
     */
    private void smsAutomaticSend(User user, Delivery delivery,boolean whiteUser) {
        if (user.getSmsSendType() != null && user.getSmsSendType() != SmsSendStatusConstants.SMS_SEND_TYPE_HAND && !whiteUser) {
            if(("CaiNiao-B").equals(delivery.getPlatform())){
                delivery.setPlatform(null);
            }
            smsService.sendSms(delivery, user);
        }
    }

    /**
     * 手机号同步
     *
     * @param filter
     * @param storeId
     * @param phone
     */
    private void phoneLibPush(DeliveryFilter filter, Integer storeId, String phone) {
        PhoneLibDto phoneLibDto = PhoneLibDto.builder()
                .storeId(storeId)
                .consingeeName(filter.getConsingeeName())
                .consingeePhone(phone.trim())
                .address(filter.getAddress())
                .buildingNumber(filter.getBuildingNumber())
                .floorNumber(filter.getFloorNumber())
                .remark(filter.getRemark())
                .phones(phone.trim())
                .build();
        phoneLibDto.setCreateTime(new Date());
        rabbitService.phoneLibPush(rabbitConfig.getPhoneExchangeName(),rabbitConfig.getPhoneRoutingKey(),JSON.toJSONString(phoneLibDto));
    }

    private void insertDeliveryScanBatch(Integer storeId, StorageDeliveryFilter storageDeliveryFilter) {
        DeliveryScanBatch scanBatch = DeliveryScanBatch.builder()
                .expCode(storageDeliveryFilter.getExpCode())
                .revexpCode(StringsUtil.stringReverse(storageDeliveryFilter.getExpCode()))
                .expId(storageDeliveryFilter.getExpId())
                .batchNumber(0)
                .storeId(storeId)
                .operatorId(1)
                .userName("")
                .userPhone(storageDeliveryFilter.getTelPhone())
                .revuserPhone(storageDeliveryFilter.getTelPhone() != null ? StringsUtil.stringReverse(storageDeliveryFilter.getTelPhone()) : null)
                .chargeMoney(null)
                .ocrStatus("0")
                .ocrAmount("0")
                .shelfCode("")
                .build();
        scanBatch.setCreateTime(new Date());
        deliveryScanBatchService.insert(scanBatch);
        //插入总表
        deliveryAsyncService.insertTotalDeliveryScanBatch(scanBatch);
    }

    private void insertDeliveryStatus(Integer storeId, String userName, Integer deliveryId) {
        DeliveryStatus deliveryStatus = DeliveryStatus.builder()
                .deliveryId(deliveryId)
                .source(ClientTypeConstants.CLIENT_TYPE_STORE_LITE_2)
                .processStaff(userName)
                .status(DeliveryStatusConstants.ORDER_STATUS_STORED)
                .preStatus(0)
                .storeId(storeId)
                .build();
        deliveryStatus.setUpdateTime(new Date());
        deliveryStatusService.insert(deliveryStatus);
        //插入总表
//        deliveryAsyncService.insertTotalDeliveryStatus(deliveryStatus);
    }

    private Delivery insertDelivery(Store store, StorageDeliveryFilter storageDeliveryFilter, boolean whiteUser, String odeliveryType, String userId) {
        Delivery delivery = new Delivery();
        delivery.setStoreId(store.getId());
        delivery.setExpcode(storageDeliveryFilter.getExpCode());
        delivery.setRevexpCode(StringsUtil.stringReverse(storageDeliveryFilter.getExpCode()));
        delivery.setExpId(storageDeliveryFilter.getExpId());
        delivery.setShelfCode(storageDeliveryFilter.getShelfCode());
        delivery.setRevshelfCode(StringsUtil.stringReverse(storageDeliveryFilter.getShelfCode()));
        delivery.setShelfId(storageDeliveryFilter.getShelfId());
        if (storageDeliveryFilter.getTelPhone() != null){
            delivery.setUserPhone(storageDeliveryFilter.getTelPhone());
            delivery.setRevuserPhone(StringsUtil.stringReverse(storageDeliveryFilter.getTelPhone()));
        }
        delivery.setChargeMoney(0);

        Date time = new Date();
        delivery.setCreateTime(time);
        delivery.setStatusTime(time);

        delivery.setStatus(DeliveryStatusConstants.ORDER_STATUS_STORED);
        delivery.setOdeliveryType(odeliveryType);
        delivery.setSignDtm(new Date());
        delivery.setUploadDtm(new Date());
        //未签收
        delivery.setSignStatus(0);
        delivery.setWhiteUser(whiteUser);
        delivery.setSignWay(1);
        delivery.setPlatform(storageDeliveryFilter.getPlatform());
        delivery.setDeliveryUname(userId);
        // 插入数据库
        deliverysDao.insert(delivery);
        Delivery newDelivery = new Delivery();
        BeanUtils.copyProperties(delivery,newDelivery);
        //插入总表
        CompletableFuture<Integer> future = deliveryAsyncService.insertTotalDelivery(delivery);
        // 等待总表插入完成时调用（需要总表ID）
        future.thenAcceptAsync(item->{
            // 快递公司消费记录
            expressConsumeService.insertConsumeRecord(delivery, ExpressAccountConstants.DEDUCTION_NODE_STORAGE);
        });
        return newDelivery;
    }



}
