package com.parksong.services.express.impl;

import com.parksong.basics.util.constants.ExpressAccountConstants;
import com.parksong.beans.delivery.Delivery;
import com.parksong.beans.express.ExpressAccountConsumeRecord;
import com.parksong.beans.express.ExpressDeptAccount;
import com.parksong.dao.express.ExpressAccountConsumeRecordDao;
import com.parksong.services.delivery.DeliverysService;
import com.parksong.services.express.ExpressConsumeService;
import com.parksong.services.express.ExpressDeptAccountService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName ExpressConsumeServiceImpl.java
 * @Description TODO
 * @createTime 2019年11月27日 16:30:00
 */
@Service
public class ExpressConsumeServiceImpl implements ExpressConsumeService {

    @Autowired
    private ExpressDeptAccountService expressDeptAccountService;

    @Autowired
    private ExpressAccountConsumeRecordDao consumeRecordDao;

    @Autowired
    private DeliverysService deliverysService;

    @Override
    public void insertConsumeRecord(Delivery delivery, byte deductionNode) {
        // 根据storeId, expId 获取快递账户
        ExpressDeptAccount account = expressDeptAccountService.getAccount(delivery.getStoreId(), delivery.getExpId());
        // 插入一条扣费记录
        if (null != account && account.getDeductionNode().compareTo(deductionNode) == 0) {
            // 出库时扣费
            if (deductionNode == ExpressAccountConstants.DEDUCTION_NODE_COMPLETE) {
                // 获取总表ID
                Integer masterId = deliverysService.queryIdFromMasterDelivery(delivery);
                if (null != masterId) {
                    delivery.setId(masterId);
                }
            }
            consumeRecordDao.insertSelective(createRecord(delivery, account));
        }
    }

    @Override
    public boolean checkExpressBalance(Integer storeId, Integer expId) {
        // 根据storeId, expId 获取快递账户
        ExpressDeptAccount account = expressDeptAccountService.getAccount(storeId, expId);
        if (null != account && account.getBalance().compareTo(account.getClosingPrice()) <= 0) {
            return false;
        }
        return true;
    }

    @Override
    public void expressChanged(Integer oldExpId, Delivery delivery) {
        ExpressAccountConsumeRecord record = consumeRecordDao.selectLastRecord(delivery.getId(), oldExpId);
        List<ExpressAccountConsumeRecord> records = new ArrayList<>();
        if (null != record && record.getType().equals(ExpressAccountConstants.CONSUME_TYPE_DEDUCTION)) {
            // 退款记录
            record.setType(ExpressAccountConstants.CONSUME_TYPE_REFUND);
            record.setRemark("修改快递公司退款 - CONSUME_ID:" + record.getId());
            records.add(record);
        }

        // 对更改后的快递公司进行扣款
        ExpressDeptAccount account = expressDeptAccountService.getAccount(delivery.getStoreId(), delivery.getExpId());
        if (null != account && account.getDeductionNode().byteValue() == ExpressAccountConstants.DEDUCTION_NODE_STORAGE) {
            records.add(createRecord(delivery, account));
        }

        if (null != records && records.size() > 0) {
            consumeRecordDao.insertList(records);
        }
    }

    private ExpressAccountConsumeRecord createRecord(Delivery delivery, ExpressDeptAccount account) {
        ExpressAccountConsumeRecord record = new ExpressAccountConsumeRecord();
        record.setStoreId(delivery.getStoreId().longValue());
        record.setExpId(delivery.getExpId().longValue());
        record.setDeliveryId(delivery.getId().longValue());
        record.setExpCode(delivery.getExpcode());

        record.setAccountId(account.getId().longValue());
        record.setDeductionNode(account.getDeductionNode());
        record.setPackageCount(1);
        record.setPackagePrice(account.getPackagePrice());
        record.setType(ExpressAccountConstants.CONSUME_TYPE_DEDUCTION);
        return record;
    }
}
