package com.parksong.services.sms.impl;

import com.alibaba.fastjson.JSON;
import com.parksong.basics.beans.BatchDataMsg;
import com.parksong.basics.exception.BusinessException;
import com.parksong.basics.services.BasicsService;
import com.parksong.basics.services.RabbitService;
import com.parksong.basics.util.configurations.RabbitConfig;
import com.parksong.basics.util.constants.Constants;
import com.parksong.basics.util.constants.YesNoConstants;
import com.parksong.basics.util.redis.CacheTool;
import com.parksong.beans.delivery.Delivery;
import com.parksong.beans.sms.DeliveryDto;
import com.parksong.beans.sms.SmsRecord;
import com.parksong.beans.sms.SmsSendQueue;
import com.parksong.beans.user.User;
import com.parksong.dao.sms.SmsDao;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.filter.sms.SmsFilter;
import com.parksong.services.delivery.DeliveryRunningWaterService;
import com.parksong.services.delivery.DeliverysService;
import com.parksong.services.sms.SmsService;
import com.parksong.services.store.ConfigurationService;
import com.parksong.util.RegexUtil;
import com.parksong.util.bacth.BatchDataHandleUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName SmsServiceImpl
 * @Description 短信
 * @Author kael
 * @Date 2019/9/27 0027 上午 9:45
 * @Version 1.0
 **/
@Service
@Slf4j
public class SmsServiceImpl extends BasicsService implements SmsService {

    @Autowired
    private SmsDao smsDao;

    @Autowired
    private ConfigurationService configurationService;

    @Autowired
    private DeliverysService deliverysService;

    @Autowired
    private RabbitService rabbitService;

    @Autowired
    private RabbitConfig rabbitConfig;

    @Autowired
    private DeliveryRunningWaterService deliveryRunningWaterService;

    @Override
    public List<BatchDataMsg> batchSendSms(SmsFilter filter) {
        List<BatchDataMsg> batchDataMsgs = new ArrayList<>();
        User user = filter.getUser();
        Map<Integer, Delivery> deliveryMap = deliverysService.queryMapByDeliveryIds(filter.getDeliveryIds(), filter.getStoreId());
        for (Integer deliveryId : filter.getDeliveryIds()) {
            Delivery delivery = deliveryMap.get(deliveryId);
            try {
                this.sendSms(delivery, user);
                batchDataMsgs.add(BatchDataHandleUtil.batchSuccess(deliveryId.toString()));
            } catch (BusinessException e) {
                batchDataMsgs.add(BatchDataHandleUtil.batchError(deliveryId.toString(), e.getMessage()));
                e.printStackTrace();
            } catch (Exception e) {
                batchDataMsgs.add(BatchDataHandleUtil.batchError(deliveryId.toString(), "操作失败"));
                e.printStackTrace();
            }
        }
        return batchDataMsgs;
    }

    @Override
    public void sendSms(Delivery delivery, User user) {
        //这段代码会引起临时计算失误，但最后统计结果还是没有问题
        if ((user.getIsSmsFree() == null || user.getIsSmsFree() == YesNoConstants.NO)) {
            if (user.getSmsBalance() == null) {
                throw new BusinessException("余额不足，请先充值");
            }
            BigDecimal consume = user.getSmsBalance().subtract(user.getSmsPrice());
            if (consume.compareTo(BigDecimal.ZERO) <= 0) {
                throw new BusinessException("余额不足，请先充值");
            } else {
                user.setSmsBalance(consume);
            }
        }
        //根据短信业务表expId,expCode,userPhone构建唯一的key
        String flagKey = delivery.getExpId() + "_" + delivery.getExpcode() + "_" + delivery.getUserPhone() + "_" + delivery.getStoreId();
        CacheTool cacheTool = CacheTool.initSMSFlag();
        String stringKey = cacheTool.getStringKey(flagKey);
        if (stringKey == null) {
            //获取过期时间
            Integer smsFlagData = Integer.parseInt(configurationService.getConfigurationByCode("SMSFlag").getOValue1());
            cacheTool.setStringandDatakey(flagKey, "0", smsFlagData * 60 * 60);
            //新增初始化短信表
            Integer count = insertSmsRecordByDelivery(delivery);
            if (count > 0) {
                // 修改业务表中的短信发送状态为发送中
                this.updateSingleDeliverySmsStatus(delivery, 0, user.getStoreId(), user.getUserId());
                // 将delivery拼接成dto用于发送短信mq
                DeliveryDto deliveryDto = getDeliveryDtoToSendSms(delivery);
                // 发送短信消息mq
                sendSms(user, deliveryDto);
            }
        } else {
            throw new BusinessException("订单已发送短信");
        }

    }

    /**
     * 将delivery拼接成dto用于发送短信mq
     *
     * @param delivery
     * @return
     */
    private DeliveryDto getDeliveryDtoToSendSms(Delivery delivery) {
        return DeliveryDto.builder()
                .delivery(delivery)
                .smsStatus(delivery.getSmsStatus() == null ? -1 : delivery.getSmsStatus())
                .sendSuccessType(delivery.getSendSuccessType())
                .resultInfo(delivery.getResultInfo())
                .sortTime(delivery.getCreateTime())
                .build();
    }

    /**
     * 发送短信消费者mq
     *
     * @param user
     * @param deliveryDto
     */
    private void sendSms(User user, DeliveryDto deliveryDto) {
        //Mq推送发短信消息
        SmsSendQueue smsSendQueue = new SmsSendQueue();
        smsSendQueue.setDeliveryDtos(deliveryDto);
        smsSendQueue.setUser(user);
        String toJson = JSON.toJSONString(smsSendQueue);

        //获取发送短信的类型(立即发送，延时发送)
        Integer smsType = user.getSmsSendType();
        Long time = Long.parseLong(configurationService.getConfigurationByCode("time" + smsType).getOValue1()) * 1000 * 60;
        log.info("【短信发送mq，smsType：{}，time：{}】",smsType,time);

        if (smsType == 1 || smsType == 2) {
            rabbitService.timingSend(rabbitConfig.getSmsQueueName(), 0L, toJson);
        } else {
            rabbitService.timingSend(rabbitConfig.getSmsQueueName(), time, toJson);
        }
        log.info("【短信发送mq，成功】");
    }

    /**
     * 修改业务表中的短信发送状态为发送中
     *
     * @param delivery
     * @param smsStatus
     * @param storeId
     * @param userId
     * @return
     */
    private int updateSingleDeliverySmsStatus(Delivery delivery, Integer smsStatus, Integer storeId, String userId) {
        List<Delivery> ids = new ArrayList<>();
        ids.add(delivery);
        return this.updateDeliverySmsStatus(ids, smsStatus, storeId, userId);
    }

    /**
     * 更新订单短信发送状态
     *
     * @param deliveryList
     * @param smsStatus
     * @param storeId
     * @param userId
     * @return
     */
    private int updateDeliverySmsStatus(List<Delivery> deliveryList, Integer smsStatus, Integer storeId, String userId) {
        if (deliveryList == null || deliveryList.isEmpty() || smsStatus == null || storeId == null) {
            return 0;
        }
        List<Integer> ids = new ArrayList<>();
        for (Delivery delivery : deliveryList) {
            ids.add(delivery.getId());
        }
        DeliveryFilter filter = new DeliveryFilter();
        filter.setStoreId(storeId);
        filter.setDeliveryIds(ids);
        filter.setSmsStatus(smsStatus);
        int count = deliverysService.updateDeliverySmsStatus(filter);

        if (count > 0) {
            for (Delivery delivery : deliveryList) {
                //新增订单流水
                deliveryRunningWaterService.updateDeliveryInsert(delivery, storeId, userId, "修改：smsStatus字段，原值：" + delivery.getSmsStatus() + "，新值：" + smsStatus,"","");
            }
        }
        return count;
    }

    public Integer insertSmsRecordByDelivery(Delivery delivery) {
        if (!RegexUtil.isValidPhoneNumberCN(delivery.getUserPhone()) && !Constants.DEFAULT_MOBILE_KB.equals(delivery.getUserPhone())) {
            throw new IllegalArgumentException("发送短信的用户手机号码不符合规则");
        }
        SmsRecord smsRecord = new SmsRecord();
        smsRecord.setStoreid(delivery.getStoreId());
        smsRecord.setDeliveryid(delivery.getId());
        smsRecord.setExpid(delivery.getExpId());
        smsRecord.setExpcode(delivery.getExpcode());
        smsRecord.setTelphone(delivery.getUserPhone());
        smsRecord.setStatus(0);
        //初始化短信记录短信价格为0
        smsRecord.setSmsPrice(new BigDecimal(0));
        smsRecord.setCreatetime(new Date(System.currentTimeMillis()));
        return smsDao.insert(smsRecord);
    }

}
