package com.parksong.services.store.impl;

import com.parksong.basics.exception.AskException;
import com.parksong.basics.exception.BusinessException;
import com.parksong.basics.services.BasicsService;
import com.parksong.beans.delivery.Delivery;
import com.parksong.beans.store.PickupQueue;
import com.parksong.dao.store.PickupQueueDao;
import com.parksong.filter.delivery.DeliveryFilter;
import com.parksong.filter.store.PickupQueueFilter;
import com.parksong.services.delivery.DeliverysService;
import com.parksong.services.store.PickupQueueService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 〈功能概述〉<br>
 *
 * @className: PickupQueueServiceImpl
 * @package: com.parksong.services.store.impl
 * @author: kael
 * @date: 2019/11/6 0006 上午 11:05
 */
@Service
public class PickupQueueServiceImpl extends BasicsService implements PickupQueueService {

    @Autowired
    private PickupQueueDao pickupQueueDao;

    @Autowired
    private DeliverysService deliverysService;

    @Override
    public List<PickupQueue> getList(PickupQueueFilter filter){
        List<PickupQueue> pickupQueues = pickupQueueDao.all(filter.getStoreId());
        pickupQueues.forEach(pickupQueue -> pickupQueue.setExpCount(pickupQueue.getExpCount()-pickupQueue.getSignCount()));
        return pickupQueues;
    }

    @Override
    public List<String> locking(PickupQueueFilter filter){
        List<String> strings = new ArrayList<>();
        if(filter.getId() != null){
            PickupQueue pickupQueue = pickupQueueDao.getById(filter.getId());
            if(pickupQueue == null){
                throw new AskException("快递已出库");
            }
            if(pickupQueue.getStatus().compareTo(2)==0){
                throw new BusinessException("找件中");
            }
            pickupQueue.setStatus(2);
            if(pickupQueueDao.update(pickupQueue)>0){
                strings.add(pickupQueue.getUserPhone());
            }else{
                throw new BusinessException("找件中");
            }
            return strings;
        }else{
            List<PickupQueue> pickupQueues = pickupQueueDao.getByQty(filter);
            if(pickupQueues == null||pickupQueues.size()==0){
                throw new AskException("无可领取的订单");
            }
            pickupQueues.forEach(pickupQueue -> {
                pickupQueue.setStatus(2);
                if(pickupQueueDao.update(pickupQueue)>0){
                    strings.add(pickupQueue.getUserPhone());
                }
            });
            return strings;
        }
    }

    @Override
    public List<Delivery> delivers(PickupQueueFilter filter){
        DeliveryFilter deliveryFilter = new DeliveryFilter();
        deliveryFilter.setStoreId(filter.getStoreId());
        deliveryFilter.setPhones(filter.getPhones());
        //判断是否为签收联想查询为出库快递信息
        if ("lenovo".equals(filter.getType())){
            deliveryFilter.setSearchExpTime(LocalDate.now().plusDays(-6).toString() + " 0:00:00");
        }
        return deliverysService.getDeliveryByPhones(deliveryFilter);
    }

    @Override
    public boolean pickupSign(Map<String, Integer> pickup){
        List<PickupQueue> pickupQueues = new ArrayList<>();
        pickup.forEach((key, value) -> {
            PickupQueue pickupQueue = pickupQueueDao.getByPhone(key);
            if(pickupQueue==null){
                return;
            }
            pickupQueue.setSignCount(pickupQueue.getSignCount()+value);
            if(pickupQueue.getSignCount().compareTo(pickupQueue.getExpCount())>=0){
                pickupQueueDao.deleteById(pickupQueue.getId());
                return;
            }
            pickupQueue.setStatus(2);
            pickupQueues.add(pickupQueue);
        });
        if(pickupQueues.size()==0){
            return true;
        }
        return pickupQueueDao.updateSignStatus(pickupQueues)>0;
    }

    @Override
    public boolean delete(PickupQueueFilter filter){
        if(filter.getIds()==null||filter.getIds().size()==0){
            return false;
        }
        return pickupQueueDao.deleteBatchIds(filter.getIds())>0;
    }
}
