package com.parksong.services.user.impl;

import com.parksong.basics.exception.BusinessException;
import com.parksong.basics.exception.NotLoginException;
import com.parksong.basics.services.BasicsService;
import com.parksong.basics.util.basic.CookieUtil;
import com.parksong.basics.util.basic.Md5Util;
import com.parksong.basics.util.constants.CookieConstants;
import com.parksong.basics.util.redis.CacheTool;
import com.parksong.beans.store.Store;
import com.parksong.beans.user.LoginData;
import com.parksong.beans.user.User;
import com.parksong.dao.store.StoreDao;
import com.parksong.dao.user.UsersDao;
import com.parksong.filter.user.UserFilter;
import com.parksong.services.express.ExpressService;
import com.parksong.services.store.AuditService;
import com.parksong.services.store.StoreService;
import com.parksong.services.user.UsersService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @ClassName UserServiceImpl
 * @Description 用户
 * @Author kael
 * @Date 2019/9/9 0009 上午 11:24
 * @Version 1.0
 **/
@Slf4j
@Service
public class UsersServiceImpl extends BasicsService implements UsersService {

    @Autowired
    private UsersDao usersDao;

    @Autowired
    private StoreService storeService;

    @Autowired
    private ExpressService expressService;

    @Value("${fastdfs.tracker_http}")
    private String trackerHttp;

    @Override
    public LoginData login(UserFilter filter){
        User user = filter.getUser();
        Store store = storeService.getStoreById(filter.getStoreId());
        //未审核
        if(store!=null&&store.getAuditStatus() == 0) {
            throw new NotLoginException("未审核");
        }
        //审核未通过
        if(store!=null&&store.getAuditStatus() == 2) {
            throw new BusinessException("审核未通过");
        }
        String md5Pwd = Md5Util.md5AsLowerHex(filter.getPassword());
        CookieUtil.setCookie(CookieConstants.COOKIE_USERID,user.getUserId());
        CookieUtil.setCookie(CookieConstants.COOKIE_KEY,md5Pwd);
        return LoginData.builder()
                .userId(user.getUserId())
                .userName(user.getUserName())
                .password(md5Pwd)
                .phoneLogo(trackerHttp + user.getPhoneLogo())
                .shortAddress(store.getShortAddress())
                .detailAddress(store.getAddress())
                .tellPhone(store.getTakeExpressPhone())
                .startTime(store.getStartTime())
                .endTime(store.getEndTime())
                .email(user.getUserMail())
                .shortName(store.getShortName())
                .status(user.getStatus())
                .smsPrice(user.getSmsPrice())
                .codeType(user.getCodeType())
                .storeId(user.getStoreId())
                .expressRuleMap(expressService.allExpressRuleMap())
                .build();
    }

    /**
     * 首先从缓存获取用户信息，若缓存没有命中，则从数据库中获取，同时刷新缓存。
     *
     * @param userId
     * @return 用户不存在时返回null
     */
    @Override
    public User getUserById(String userId) {

        CacheTool cacheTool = CacheTool.initUser();
        User user = (User) cacheTool.getBean(userId);
        if (user != null && user.getStatus()!=null && (user.getStatus()==1||user.getStatus()==3)) {
            return user;
        } else {
            user = usersDao.getUserByIdAndPwd(UserFilter.builder()
                    .userId(userId).build());
            log.info("从远程接口获取用户信息:" + userId);
            if (user != null) {
                cacheTool.saveBean(userId, user);
                log.info("刷新用户memcached:" + userId);
                return user;
            }
            return null;
        }

    }
}
