package com.parksong.util;

import com.parksong.beans.delivery.StatisticsOutboundRate;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.VerticalAlignment;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author LiuJS
 * @Time 2021/6/17 10:24
 * @Desc excel 工具类
 * @Version 1.0
 */
public class ExcelUtil {

    public static void exportExpressData(HttpServletResponse response, List<StatisticsOutboundRate> totalList) {
        OutputStream ouputStream = null;
        try {
            String workbookName = "驿站每日快递出入库数据统计";
            String fileName = URLEncoder.encode(workbookName, "UTF-8");
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xls");

            String[] totalTitle = {"日期","入库数","出库数","待出库"};
            String[] branchTitle = {"快递公司","入库数","出库数","待出库"};

            Map<String, List<StatisticsOutboundRate>> branchListMap = new LinkedHashMap<>();
            totalList.stream().forEach(statisticsOutboundRate -> branchListMap.put(statisticsOutboundRate.getTime(), statisticsOutboundRate.getDetailInfo()));

            //创建excel表
            HSSFWorkbook wb = new HSSFWorkbook();
            //写入汇总数据
            writeDate(wb, "汇总", totalTitle, true, totalList);
            //写入详细数据
            for (Map.Entry<String, List<StatisticsOutboundRate>> branch :branchListMap.entrySet()) {
                writeDate(wb, branch.getKey(), branchTitle, false, branch.getValue());
            }
            ouputStream = response.getOutputStream();
            wb.write(ouputStream);
            ouputStream.flush();
            ouputStream.close();
        }catch (Exception e) {
            Logger.getLogger(ExcelUtil.class.getName()).log(Level.SEVERE, null, e);
            e.printStackTrace();
            try {
                ouputStream.close();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }

    private static void writeDate(HSSFWorkbook wb, String sheetName, String[] title, boolean result, List<StatisticsOutboundRate> list) {
        //表头样式（加粗，水平居中，垂直居中）
        HSSFCellStyle cellStyle = wb.createCellStyle();
        //垂直居中
        cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        HSSFFont fontStyle = wb.createFont();
        fontStyle.setBold(true);
        cellStyle.setFont(fontStyle);
        //标题样式（加粗，垂直居中）
        HSSFCellStyle cellStyle2 = wb.createCellStyle();
        //垂直居中
        cellStyle2.setVerticalAlignment(VerticalAlignment.CENTER);
        cellStyle2.setFont(fontStyle);
        //字段样式（垂直居中）
        HSSFCellStyle cellStyle3 = wb.createCellStyle();
        cellStyle3.setVerticalAlignment(VerticalAlignment.CENTER);

        HSSFCell hc;
        HSSFSheet sheet = wb.createSheet(sheetName);
        //设置默认行宽
        sheet.setDefaultColumnWidth(20);
        //创建标题
        HSSFRow rowTitle = sheet.createRow(0);
        rowTitle.setHeightInPoints(20);
        for (int i = 0; i < title.length; i++) {
            hc = rowTitle.createCell(i);
            hc.setCellValue(title[i]);
            hc.setCellStyle(cellStyle2);
        }

        try {
            //创建表格数据
            int i = 1;
            StatisticsOutboundRate total = StatisticsOutboundRate.builder()
                    .warehousing(0)
                    .outboundNum(0)
                    .waitOutboundNum(0)
                    .build();
            for (StatisticsOutboundRate statisticsOutboundRate : list) {
                HSSFRow rowBody = sheet.createRow(i);
                rowBody.setHeightInPoints(20);
                hc = rowBody.createCell(0);
                if (result){
                    hc.setCellValue(statisticsOutboundRate.getTime());
                    total.setWarehousing(total.getWarehousing() + statisticsOutboundRate.getWarehousing());
                    total.setOutboundNum(total.getOutboundNum() + statisticsOutboundRate.getOutboundNum());
                }else {
                    hc.setCellValue(statisticsOutboundRate.getExpName());
                }

                hc = rowBody.createCell(1);
                hc.setCellValue(statisticsOutboundRate.getWarehousing());
                hc = rowBody.createCell(2);
                hc.setCellValue(statisticsOutboundRate.getOutboundNum());
                hc = rowBody.createCell(3);
                hc.setCellValue(statisticsOutboundRate.getWaitOutboundNum());
                i++;
            }
            if (result){
                HSSFRow rt = sheet.createRow(i);
                rt.setHeightInPoints(20);
                hc = rt.createCell(0);
                hc.setCellValue("合计");
                hc.setCellStyle(cellStyle2);
                hc = rt.createCell(1);
                hc.setCellValue(total.getWarehousing());
                hc.setCellStyle(cellStyle2);
                hc = rt.createCell(2);
                hc.setCellValue(total.getOutboundNum());
                hc.setCellStyle(cellStyle2);
            }

        } catch (Exception e) {
            Logger.getLogger(ExcelUtil.class.getName()).log(Level.SEVERE, null, e);
        } finally {
            try {
                wb.close();
            } catch (IOException e) {
                Logger.getLogger(ExcelUtil.class.getName()).log(Level.SEVERE, null, e);
            }
        }
    }
}
