package com.parksong.util;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTCreator;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.parksong.basics.exception.BusinessException;
import com.parksong.basics.exception.NotLoginException;
import org.apache.catalina.authenticator.NonLoginAuthenticator;
import org.apache.commons.lang3.time.DateUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author LiuJS
 * @Time 2021/7/6 16:31
 * @Desc
 * @Version 1.0
 */
public class JwtUtils {

    private static final String SECRET = "youbaidi-2021";
    private static final String ISSUER = "ybm";

    /**
     * 生成token
     * @param claims
     * @return
     * @throws Exception
     */
    public static String createToken(Map<String, String> claims) throws Exception {
        try {
            Algorithm algorithm = Algorithm.HMAC256(SECRET);
            JWTCreator.Builder builder = JWT.create()
                    .withIssuer(ISSUER)
                    //设置过期时间为2小时
                    .withExpiresAt(DateUtils.addHours(new Date(), 2));
            claims.forEach(builder::withClaim);
            return builder.sign(algorithm);
        }catch (IllegalArgumentException  e) {
            throw new Exception("生成token失败");
        }
    }

    /**
     * 校验token,并返回数据
     * @param token
     * @return
     */
    public static Map<String, String> verifyToken(String token) throws Exception {
        Algorithm algorithm;
        Map<String, Claim> map;
        try {
            algorithm = Algorithm.HMAC256(SECRET);
            JWTVerifier verifier = JWT.require(algorithm).withIssuer(ISSUER).build();
            DecodedJWT jwt = verifier.verify(token);
            map = jwt.getClaims();
        } catch (Exception e) {
            throw new NotLoginException("Token鉴权失败,请重新登录");
        }
        Map<String, String> resultMap = new HashMap<>(map.size());
        map.forEach((k, v) -> resultMap.put(k, v.asString()));
        return resultMap;
    }
}
