package com.parksong.util.txt;

import org.springframework.util.ResourceUtils;
import org.springframework.util.StringUtils;

import java.io.*;
import java.util.List;

/**
 * @ClassName TXTUtils
 * @Description 文本工具
 * @Author kael
 * @Date 2019/9/23 0023 上午 9:53
 * @Version 1.0
 **/
public class TxtUtils {
    /**
     * 截取字符串
     * @param str
     * @param num
     * @return
     */
    public static String subString(String str,Integer num){
        if(com.google.common.base.Strings.isNullOrEmpty(str)){
            return "";
        }
        if(str.length() < num){
            return str;
        }
        return str.substring(str.length()- num,str.length());
    }

    public static void writeToTxt(String str, String path,String fileName){
        try {
            String basePath = getResourceBasePath();
            String studentResourcePath = new File(path, "guoanFile/"+fileName).getAbsolutePath();
            // 保证目录一定存在
            ensureDirectory(studentResourcePath);
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(studentResourcePath)));
            writer.write(str);
            writer.flush();
            writer.close();
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    /**
     * 获取项目根路径
     *
     * @return
     */
    public static String getResourceBasePath() {
        // 获取跟目录
        File path = null;
        try {
            path = new File(ResourceUtils.getURL("classpath:").getPath());
        } catch (FileNotFoundException e) {
            // nothing to do
        }
        if (path == null || !path.exists()) {
            path = new File("");
        }

        String pathStr = path.getAbsolutePath();
        // 如果是在eclipse中运行，则和target同级目录,如果是jar部署到服务器，则默认和jar包同级
        pathStr = pathStr.replace("\\target\\classes", "");

        return pathStr;
    }

    /**
     * 保证拷贝的文件的目录一定要存在
     *
     * @param filePath
     *            文件路径
     */
    public static void ensureDirectory(String filePath) {
        if (StringUtils.isEmpty(filePath)) {
            return;
        }
        filePath = replaceSeparator(filePath);
        if (filePath.indexOf("/") != -1) {
            filePath = filePath.substring(0, filePath.lastIndexOf("/"));
            File file = new File(filePath);
            if (!file.exists()) {
                file.mkdirs();
            }
        }
    }

    /**
     * 将符号“\\”和“\”替换成“/”,有时候便于统一的处理路径的分隔符,避免同一个路径出现两个或三种不同的分隔符
     *
     * @param str
     * @return
     */
    public static String replaceSeparator(String str) {
        return str.replace("\\", "/").replace("\\\\", "/");
    }
}
